@@ -24,6 +24,7 @@
 #define __LIBBALSA_ADDRESS_H__
 
 #include <gtk/gtk.h>
+#include <gmime/gmime.h>
 
 #define LIBBALSA_TYPE_ADDRESS				(libbalsa_address_get_type())
 #define LIBBALSA_ADDRESS(obj)				(G_TYPE_CHECK_INSTANCE_CAST (obj, LIBBALSA_TYPE_ADDRESS, LibBalsaAddress))
@@ -39,9 +40,6 @@ typedef enum _LibBalsaAddressField LibBalsaAddressField;
 enum _LibBalsaAddressField {
     FULL_NAME,
     FIRST_NAME,
-#if !defined(ENABLE_TOUCH_UI)
-    MIDDLE_NAME,
-#endif
     LAST_NAME,
     NICK_NAME,
     ORGANIZATION,
@@ -49,8 +47,7 @@ enum _LibBalsaAddressField {
     NUM_FIELDS
 };
 
-/* General address structure. it should possible subclass more compact
-   rfc2822_mailbox, or something.
+/* General address structure to be used with address books.
 */
 struct _LibBalsaAddress {
     GObject parent;
@@ -63,17 +60,16 @@ struct _LibBalsaAddress {
     gchar *nick_name;
 
     /* First and last names
-     * VCard: N: field
-     * LDAP/LDIF: cn
-     * Full name is the bit in <> in an rfc822 address
+     * VCard: parsed from N: field
+     * LDAP/LDIF: cn, givenName, surName.
      */
     gchar *full_name;
     gchar *first_name;
-    gchar *middle_name;
     gchar *last_name;
 
     /* Organisation
      * VCard: ORG: field
+     * ldif: o: attribute.
      */
     gchar *organization;
 
@@ -90,36 +86,38 @@ struct _LibBalsaAddressClass {
 GType libbalsa_address_get_type(void);
  
 LibBalsaAddress *libbalsa_address_new(void);
-LibBalsaAddress *libbalsa_address_new_from_string(const gchar * address);
 void libbalsa_address_set_copy(LibBalsaAddress *dest, LibBalsaAddress *src);
 gchar *libbalsa_address_to_gchar(LibBalsaAddress * address, gint n);
-GList *libbalsa_address_new_list_from_string(const gchar * address);
 
-/* get pointer to descriptive name (full name if available, or e-mail) */
-const gchar *libbalsa_address_get_name(const LibBalsaAddress * addr);
-
-/* libbalsa_address_get_mailbox and libbalsa_address_get_phrase
-   are used to create the ESMTP envelope of the message. Note that
-   they have different semantics than libbalsa_addres_to_gchar() 
-   when the address is a group. get_mailbox() must return all addresses
-   while address_to_gchar() may return empty list.
-*/
-/* XXX - added by Brian Stafford <brian@stafford.uklinux.net> */
-const gchar *libbalsa_address_get_mailbox(LibBalsaAddress * address, gint n);
-#if ENABLE_ESMTP
-/* XXX - added by Brian Stafford <brian@stafford.uklinux.net> */
-const gchar *libbalsa_address_get_phrase(LibBalsaAddress * address);
-#endif
+const gchar *libbalsa_address_get_name_from_list(const InternetAddressList
+                                                 * address_list);
+const gchar *libbalsa_address_get_mailbox_from_list(const
+                                                    InternetAddressList *
+                                                    address_list);
 
 /* =================================================================== */
 /*                                UI PART                              */
 /* =================================================================== */
 
-/** libbalsa_address_get_edit_widget() returns an widget adapted
-    for a LibBalsaAddress edition, with initial values set if address
-    is provided. The edit entries are set in entries array 
-    and enumerated with LibBalsaAddressField constants
+/** libbalsa_address_set_edit_entries() initializes the GtkEntry widgets
+    in entries with values from address
 */
+void libbalsa_address_set_edit_entries(LibBalsaAddress * address,
+                                       GtkWidget ** entries);
+/** libbalsa_address_get_edit_widget() returns an widget adapted for a
+    LibBalsaAddress edition, with initial values set if address is
+    provided. The edit entries are set in entries array and enumerated
+    with LibBalsaAddressField constants. The widget accepts drops of
+    type TARGET_ADDRESS and TARGET_STRING.
+*/
+
+enum {
+    LIBBALSA_ADDRESS_TRG_STRING,
+    LIBBALSA_ADDRESS_TRG_ADDRESS
+};
+
+extern GtkTargetEntry libbalsa_address_target_list[2];
+
 GtkWidget *libbalsa_address_get_edit_widget(LibBalsaAddress *addr,
                                             GtkWidget **entries,
                                             GCallback changed_cb,
