@@ -41,6 +41,20 @@ ImapResponse imap_mbox_subscribe(ImapMboxHandle* handle,
                                  const char* mbox, gboolean subscribe);
 ImapResponse imap_mbox_list(ImapMboxHandle *r, const char*what);
 ImapResponse imap_mbox_lsub(ImapMboxHandle *r, const char*what);
+typedef enum {
+  IMSTAT_MESSAGES = 0,
+  IMSTAT_RECENT,
+  IMSTAT_UIDNEXT,
+  IMSTAT_UIDVALIDITY,
+  IMSTAT_UNSEEN,
+  IMSTAT_NONE
+} ImapStatusItem;
+struct ImapStatusResult {
+  ImapStatusItem item;
+  unsigned       result;
+};
+ImapResponse imap_mbox_status(ImapMboxHandle *r, const char*what, 
+                              struct ImapStatusResult *res);
 typedef size_t (*ImapAppendFunc)(char*, size_t, void*);
 ImapResponse imap_mbox_append(ImapMboxHandle *handle, const char *mbox,
                               ImapMsgFlags flags, size_t sz, 
@@ -58,10 +72,14 @@ ImapResponse imap_mbox_close(ImapMboxHandle *h);
 ImapResult imap_mbox_search(ImapMboxHandle *h, const char* query);
 ImapResponse imap_mbox_noop(ImapMboxHandle *r);
 ImapResponse imap_mbox_expunge(ImapMboxHandle* h);
+ImapResponse imap_mbox_expunge_a(ImapMboxHandle *h);
 
 ImapResponse imap_mbox_store_flag(ImapMboxHandle *r, unsigned cnt,
                                   unsigned *seqno, ImapMsgFlag flg,
                                   gboolean state);
+unsigned imap_mbox_store_flag_a(ImapMboxHandle *r, unsigned cnt,
+				unsigned *seqno, ImapMsgFlag flg,
+				gboolean state);
 ImapResponse imap_mbox_handle_copy(ImapMboxHandle* handle, unsigned cnt,
                                    unsigned *seqno, const gchar *dest);
 
@@ -82,10 +100,12 @@ ImapResponse imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
                                            unsigned seqno, FILE *fl);
 ImapResponse imap_mbox_handle_fetch_rfc822_uid(ImapMboxHandle* handle,
                                                unsigned uid, FILE *fl);
-typedef void (*ImapFetchBodyCb)(const char *buf, int buflen, void* arg);
+typedef void (*ImapFetchBodyCb)(unsigned seqno, const char *buf,
+				int buflen, void* arg);
 ImapResponse imap_mbox_handle_fetch_body(ImapMboxHandle* handle, 
                                          unsigned seqno, 
                                          const char *section,
+                                         ImapFetchBodyOptions options,
                                          ImapFetchBodyCb body_handler,
                                          void *arg);
 
@@ -102,11 +122,16 @@ ImapResponse imap_mbox_uid_search(ImapMboxHandle *handle, ImapSearchKey *key,
                                   void *cb_data);
 
 ImapResponse imap_mbox_sort_msgno(ImapMboxHandle *handle, ImapSortKey key,
-                                  int ascending, int *msgno, unsigned cnt);
+                                  int ascending, unsigned int *msgno,
+				  unsigned cnt);
 ImapResponse imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key,
                                    int ascending, ImapSearchKey *filter);
 ImapResponse imap_mbox_filter_msgnos(ImapMboxHandle * handle,
                                      ImapSearchKey *filter,
                                      GHashTable * msgnos);
 
+ImapResponse imap_mbox_complete_msgids(ImapMboxHandle *handle,
+				       GPtrArray *msgids,
+				       unsigned first_seqno_to_fetch);
+
 #endif /* __IMAP_COMMANDS_H__ */
