@@ -60,9 +60,10 @@ typedef enum
   IMCAP_STATUS,
   IMCAP_ACL,			/* RFC 2086: IMAP4 ACL extension */
   IMCAP_NAMESPACE,              /* RFC 2342: IMAP4 Namespace */
+  IMCAP_AANONYMOUS, 	        /* RFC 2245: AUTH=ANONYMOUS */
   IMCAP_ACRAM_MD5,		/* RFC 2195: CRAM-MD5 authentication */
   IMCAP_AGSSAPI,		/* RFC 1731: GSSAPI authentication */
-  IMCAP_AUTH_ANON, 	        /*           AUTH=ANONYMOUS */
+  IMCAP_APLAIN,                 /* RFC 2595: */
   IMCAP_STARTTLS,		/* RFC 2595: STARTTLS */
   IMCAP_LOGINDISABLED,		/*           LOGINDISABLED */
   IMCAP_SORT,                   /* SORT and THREAD described at: */
@@ -73,6 +74,7 @@ typedef enum
   IMCAP_SCAN,                   /* FIXME: RFC? */
   IMCAP_CHILDREN,               /* RFC 3348 */
   IMCAP_LITERAL,                /* RFC 2088 */
+  IMCAP_IDLE,                   /* RFC 2177 */
   IMCAP_SASLIR,                 /* imap-sasl-initial-response: SASL-IR */
   IMCAP_FETCHBODY,              /* basic imap implemented correctly by
                                  * most imap servers but not all. We
@@ -80,6 +82,11 @@ typedef enum
   IMCAP_MAX
 } ImapCapability;
 
+typedef enum {
+  IMAP_OPT_ANONYMOUS, /* try anonymous authentication */
+  IMAP_OPT_BINARY     /* enable binary=no-transfer-encoding msg transfer */
+}  ImapOption;
+
 typedef struct _ImapMboxHandleClass ImapMboxHandleClass;
 typedef void (*LBIResponseCallback)(ImapMboxHandle *h, const char* response,
                                     void *data);
@@ -105,11 +112,14 @@ typedef void(*ImapListCb)(ImapMboxHandle*handle, int delim,
 
 
 ImapMboxHandle *imap_mbox_handle_new(void);
+void imap_handle_set_option(ImapMboxHandle *h, ImapOption opt, gboolean state);
 void imap_handle_set_monitorcb(ImapMboxHandle* h, ImapMonitorCb cb, void*);
 void imap_handle_set_infocb(ImapMboxHandle* h, ImapInfoCb cb, void*);
 void imap_handle_set_usercb(ImapMboxHandle* h, ImapUserCb cb, void*);
 void imap_handle_set_flagscb(ImapMboxHandle* h, ImapFlagsCb cb, void*);
 void imap_handle_set_timeout(ImapMboxHandle *, int milliseconds);
+gboolean imap_handle_idle_enable(ImapMboxHandle *, int seconds);
+gboolean imap_handle_idle_disable(ImapMboxHandle *);
 
 ImapResult imap_mbox_handle_connect(ImapMboxHandle* r, const char *hst, 
                                     int over_ssl);
@@ -187,11 +197,13 @@ ImapSearchKey *imap_search_key_new_string(unsigned negated,
 ImapSearchKey* imap_search_key_new_size_greater(unsigned negated, size_t sz);
 ImapSearchKey* imap_search_key_new_date(ImapSearchDateRange range,
                                         int internal, time_t tm);
+ImapSearchKey* imap_search_key_new_range(unsigned negated, int uid,
+                                         unsigned lo, unsigned hi);
 
 void imap_search_key_free(ImapSearchKey *s);
 void imap_search_key_set_next(ImapSearchKey *list, ImapSearchKey *next);
-ImapResponse imap_search_exec(ImapMboxHandle *h, ImapSearchKey *s,
-                              ImapSearchCb cb, void *cb_arg);
+ImapResponse imap_search_exec(ImapMboxHandle *h, gboolean uid, 
+                              ImapSearchKey *s, ImapSearchCb cb, void *cb_arg);
 /* ================ END OF SEARCH FUNCTIONS ============================ */
 
 /* ================ BEGIN OF MBOX_VIEW FUNCTIONS ======================= */
