@@ -125,10 +125,11 @@ void
 pop_set_option(PopHandle *pop, PopOption opt, gboolean state)
 {
   switch(opt) {
-  case IMAP_POP_OPT_DISABLE_APOP: pop->disable_apop = state; break;
-  case IMAP_POP_OPT_FILTER_CR   : pop->filter_cr    = state; break;
-  case IMAP_POP_OPT_OVER_SSL    : pop->over_ssl     = state; break;
-  case IMAP_POP_OPT_PIPELINE    : pop->enable_pipe  = state; break;
+  case IMAP_POP_OPT_DISABLE_APOP: pop->disable_apop = !!state; break;
+  case IMAP_POP_OPT_FILTER_CR   : pop->filter_cr    = !!state; break;
+  case IMAP_POP_OPT_OVER_SSL    : pop->over_ssl     = !!state; break;
+  case IMAP_POP_OPT_PIPELINE    : pop->enable_pipe  = !!state; break;
+  default: g_warning("pop_set_option: invalid option\n");
   }
 }
 
@@ -175,7 +176,7 @@ pop_check_status(PopHandle *pop, GError **err)
     pop->state = IMHS_DISCONNECTED;
     sio_detach(pop->sio); pop->sio = NULL; close(pop->sd);
     g_set_error(err, IMAP_ERROR, IMAP_POP_SEVERED_ERROR,
-                "Connection severed");
+                "POP3 Connection severed");
     return FALSE;
   }
      
@@ -238,7 +239,7 @@ pop_get_capa(PopHandle *pop, GError **err)
     pop->state = IMHS_DISCONNECTED;
     sio_detach(pop->sio); pop->sio = NULL; close(pop->sd);
     g_set_error(err, IMAP_ERROR, IMAP_POP_SEVERED_ERROR,
-                "Connection severed");
+                "POP3 Connection severed");
     return FALSE;
   }
   return TRUE;
@@ -418,9 +419,11 @@ pop_stls(PopHandle *pop, GError **err)
     pop->tls_enabled = 1;
     return TRUE;
   } else {
-    SSL_free(ssl);
+    /* this will destroy ssl, too */
     sio_detach(pop->sio); pop->sio = NULL; close(pop->sd);
     pop->state = IMHS_DISCONNECTED;
+    g_set_error(err, IMAP_ERROR, IMAP_POP_SEVERED_ERROR,
+                "TLS setup failed");
     return FALSE;
   }
 }
@@ -468,7 +471,7 @@ gboolean
 pop_connect(PopHandle *pop, const char *host, GError **err)
 {
   static const int SIO_BUFSZ=8192;
-  static const char *service = "pop3";
+  const char *service = "pop3";
   char line[POP_LINE_LEN];
 
 #ifdef USE_TLS
@@ -523,7 +526,8 @@ pop_connect(PopHandle *pop, const char *host, GError **err)
   
 #ifdef USE_TLS
   if(pop->tls_mode != IMAP_TLS_DISABLED && pop_can_do(pop, POP_CAP_STLS)) {
-    pop_stls(pop, err); /* Try... */
+    if(!pop_stls(pop, err)) /* TLS negotiation attempted.. */
+      return FALSE;         /* .. but failed. */
   }
 #endif
   if(pop->tls_mode == IMAP_TLS_REQUIRED && 
@@ -644,8 +648,10 @@ pop_destroy(PopHandle *pop, GError **err)
   while(pop->req_insert_pos && pop->state == IMHS_AUTHENTICATED)
     pop_complete_pending_requests(pop);
 
-  if(pop->sio) {
+  if(pop->sio)
     res = pop_exec(pop, "Quit\r\n", err);
+  /* check again */
+  if(pop->sio) {
     sio_detach(pop->sio); pop->sio = NULL;
     close(pop->sd);
   }
