@@ -377,7 +377,7 @@ static void
 lbm_mbox_save(LibBalsaMailboxMbox * mbox)
 {
     gchar *filename;
-#if GLIB_CHECK_VERSION(2, 8, 0)
+#if GLIB_CHECK_VERSION(2, 8, 0) && !defined(__APPLE__)
     GError *err = NULL;
 #endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
 
@@ -389,7 +389,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
     filename = lbm_mbox_get_cache_filename(mbox);
 
     if (mbox->messages_info->len > 0) {
-#if GLIB_CHECK_VERSION(2, 8, 0)
+#if GLIB_CHECK_VERSION(2, 8, 0) && !defined(__APPLE__)
         if (!g_file_set_contents(filename, mbox->messages_info->data,
                                  mbox->messages_info->len
                                  * sizeof(struct message_info), &err)) {
@@ -1054,24 +1054,25 @@ libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
                                     gboolean expunge)
 {
     LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
+    guint len;
 
-    if (mbox->messages_info) {
-	guint len;
+    len = mbox->messages_info->len;
+    libbalsa_mailbox_mbox_sync(mailbox, expunge);
+    if (mbox->messages_info->len != len)
+        libbalsa_mailbox_changed(mailbox);
 
-	len = mbox->messages_info->len;
-        libbalsa_mailbox_mbox_sync(mailbox, expunge);
-	if (mbox->messages_info->len != len)
-	    libbalsa_mailbox_changed(mailbox);
-	free_messages_info(mbox->messages_info);
-	mbox->messages_info = NULL;
-    }
     if (mbox->gmime_stream) {
-	g_object_unref(mbox->gmime_stream);
-	mbox->gmime_stream = NULL;	/* chbm: is this correct? */
+        g_object_unref(mbox->gmime_stream);
+        mbox->gmime_stream = NULL;
     }
+
     if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
         LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
                                                             expunge);
+
+    /* Now it's safe to free the message info. */
+    free_messages_info(mbox->messages_info);
+    mbox->messages_info = NULL;
 }
 
 static GMimeMessage *
