@@ -36,9 +36,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <fcntl.h>
-
 #include <glib.h>
-#include <libgnome/libgnome.h> 
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
@@ -46,6 +44,9 @@
 /* needed for truncate_string */
 #include "misc.h"
 
+#include "mime-stream-shared.h"
+#include "i18n.h"
+
 #include <gmime/gmime.h>
 
 static void libbalsa_message_class_init(LibBalsaMessageClass * klass);
@@ -82,7 +83,6 @@ libbalsa_message_get_type()
     return libbalsa_message_type;
 }
 
-int message_cnt = 0;
 static void
 libbalsa_message_init(LibBalsaMessage * message)
 {
@@ -102,7 +102,6 @@ libbalsa_message_init(LibBalsaMessage * message)
 #ifdef HAVE_GPGME
     message->prot_state = LIBBALSA_MSG_PROTECT_NONE;
 #endif
-    /* printf("%p message created.\n", message); */ message_cnt++;
 }
 
 
@@ -144,7 +143,7 @@ libbalsa_message_finalize(GObject * object)
     message->headers = NULL;
 
     if (message->sender) {
-	g_object_unref(message->sender);
+	internet_address_list_destroy(message->sender);
 	message->sender = NULL;
     }
 
@@ -179,20 +178,6 @@ libbalsa_message_finalize(GObject * object)
 	message->mime_msg = NULL;
     }
     G_OBJECT_CLASS(parent_class)->finalize(object);
-    /* printf("%p message finalized.\n", message); */  message_cnt--;
-}
-
-static void
-libbalsa_message_find_charset(GMimeObject * mime_part,
-                              const gchar ** charset)
-{
-    const GMimeContentType *type;
-
-    if (*charset)
-        return;
-
-    type = g_mime_object_get_content_type(mime_part);
-    *charset = g_mime_content_type_get_parameter(type, "charset");
 }
 
 static void
@@ -215,34 +200,37 @@ libbalsa_message_headers_destroy(LibBalsaMessageHeaders * headers)
     headers->subject = NULL;
 
     if (headers->from) {
-	g_object_unref(headers->from);
+	internet_address_list_destroy(headers->from);
 	headers->from = NULL;
     }
 
-    g_list_foreach(headers->to_list, (GFunc) g_object_unref, NULL);
-    g_list_free(headers->to_list);
-    headers->to_list = NULL;
+    if (headers->to_list) {
+	internet_address_list_destroy(headers->to_list);
+	headers->to_list = NULL;
+    }
 
     if (headers->content_type) {
 	g_mime_content_type_destroy(headers->content_type);
 	headers->content_type = NULL;
     }
 
-    g_list_foreach(headers->cc_list, (GFunc) g_object_unref, NULL);
-    g_list_free(headers->cc_list);
-    headers->cc_list = NULL;
+    if (headers->cc_list) {
+	internet_address_list_destroy(headers->cc_list);
+	headers->cc_list = NULL;
+    }
+
+    if (headers->bcc_list) {
+	internet_address_list_destroy(headers->bcc_list);
+	headers->bcc_list = NULL;
+    }
 
-    g_list_foreach(headers->bcc_list, (GFunc) g_object_unref, NULL);
-    g_list_free(headers->bcc_list);
-    headers->bcc_list = NULL;
-    
     if (headers->reply_to) {
-	g_object_unref(headers->reply_to);
+	internet_address_list_destroy(headers->reply_to);
 	headers->reply_to = NULL;
     }
 
     if(headers->dispnotify_to) {
-	g_object_unref(headers->dispnotify_to);
+	internet_address_list_destroy(headers->dispnotify_to);
 	headers->dispnotify_to = NULL;
     }
 
@@ -254,57 +242,29 @@ libbalsa_message_headers_destroy(LibBalsaMessageHeaders * headers)
 const gchar *
 libbalsa_message_body_charset(LibBalsaMessageBody * body)
 {
-    const gchar *charset = NULL;
+    const gchar *charset;
+
+    if (!body)
+	return NULL;
 
     if (body->charset) /* This overrides all! Important for non
                         * us-ascii messages over IMAP. */
         return body->charset;
-    if (body->mime_part) {
-	if (GMIME_IS_MULTIPART(body->mime_part))
-	    g_mime_multipart_foreach(GMIME_MULTIPART(body->mime_part),
-				     (GMimePartFunc)
-				     libbalsa_message_find_charset, &charset);
-	else
-	    libbalsa_message_find_charset(body->mime_part, &charset);
-    } else {
-	do {
-	    if (body->charset) {
-		charset = body->charset;
-		break;
-	    }
-	    if (body->parts)
-		charset = libbalsa_message_body_charset(body->parts);
-	} while (!charset && (body = body->next));
+
+    if (GMIME_IS_PART(body->mime_part)) {
+        const GMimeContentType *type;
+
+        type = g_mime_object_get_content_type(body->mime_part);
+        return g_mime_content_type_get_parameter(type, "charset");
     }
 
-    return charset;
+    charset = libbalsa_message_body_charset(body->parts);
+    if (charset)
+        return charset;
+
+    return libbalsa_message_body_charset(body->next);
 }
 
-/* Note: libbalsa_message_charset returns a pointer to a newly allocated
- * string containing the canonical form of the charset field, or NULL.
- * When the pointer is nonNULL, the string must be deallocated with
- * g_free. */
-gchar *
-libbalsa_message_charset(LibBalsaMessage * message)
-{
-    LibBalsaMessageBody *body;
-    const gchar *charset;
-    const char *tmp;
-    g_return_val_if_fail(message != NULL, NULL);
-    body = message->body_list;
-    g_return_val_if_fail(body != NULL, NULL);
-
-    charset = body->charset;
-    if (!charset) {
-        charset = libbalsa_message_body_charset(body);
-        if (!charset)
-            return NULL;
-    }
-    tmp = g_mime_charset_canon_name(charset);
-    return g_strdup(tmp);
-}
-
-
 static void
 canonize_header_value(gchar *value)
 {
@@ -332,7 +292,7 @@ canonize_header_value(gchar *value)
    The list has to be freed by the following chunk of code:
     FREE_HEADER_LIST(list);
 */
-gchar **
+static gchar **
 libbalsa_create_hdr_pair(const gchar * name, gchar * value)
 {
     gchar **item = g_new(gchar *, 3);
@@ -347,19 +307,18 @@ libbalsa_create_hdr_pair(const gchar * name, gchar * value)
 /** libbalsa_message_find_user_hdr:
     returns.... list element matching given header.
 */
-GList *
+static GList *
 libbalsa_message_find_user_hdr(LibBalsaMessage * message, const gchar * find)
 {
     GList* list;
-    gchar** tmp;
     LibBalsaMessageHeaders *headers = message->headers;
     
     g_return_val_if_fail(headers, NULL);
     if (!headers->user_hdrs && message->mailbox) 
         libbalsa_mailbox_set_msg_headers(message->mailbox, message);
 
-    for (list = headers->user_hdrs; list; list = g_list_next(list)) {
-        tmp = list->data;
+    for (list = headers->user_hdrs; list; list = list->next) {
+        const gchar * const *tmp = list->data;
         
         if (g_ascii_strncasecmp(tmp[0], find, strlen(find)) == 0) 
             return list;
@@ -368,9 +327,55 @@ libbalsa_message_find_user_hdr(LibBalsaMessage * message, const gchar * find)
     return NULL;
 }
 
+/* 
+ * Public user header methods
+ */
+const gchar *
+libbalsa_message_get_user_header(LibBalsaMessage * message,
+                                 const gchar * name)
+{
+    GList *header;
+    const gchar *const *pair;
+    
+    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), NULL);
+    g_return_val_if_fail(name != NULL, NULL);
+
+    if (!(header = libbalsa_message_find_user_hdr(message, name)))
+        return NULL;
+
+    pair = header->data;
+    return pair[1];
+}
+
+void
+libbalsa_message_set_user_header(LibBalsaMessage * message,
+                                 const gchar * name, const gchar * value)
+{
+    LibBalsaMessageHeaders *headers;
+    GList *header;
+
+    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
+    g_return_if_fail(name != NULL);
+
+    headers = message->headers;
+    g_return_if_fail(headers != NULL);
+
+    if ((header = libbalsa_message_find_user_hdr(message, name))) {
+        headers->user_hdrs =
+            g_list_remove_link(headers->user_hdrs, header);
+        FREE_HEADER_LIST(header);
+    }
+
+    if (value && *value)
+        headers->user_hdrs =
+            g_list_prepend(headers->user_hdrs,
+                           libbalsa_create_hdr_pair(name,
+                                                    g_strdup(value)));
+}
+
 static void
 prepend_header_misc(const char *name, const char *value,
-			 gpointer user_data)
+                    gpointer user_data)
 {
     char lcname[17]; /* one byte longer than the longest ignored header */
     static const char ignored_headers[] =
@@ -384,17 +389,18 @@ prepend_header_misc(const char *name, const char *value,
     /* Standard Headers*/
     for(i=0; name[i] && i<sizeof(lcname)-1; i++)
         lcname[i] = tolower(name[i]);
-    if(name[i]) /* too long to be on the ignored-headers list */
-        return;
-    lcname[i] = '\0';
-    if(strstr(ignored_headers, lcname))
-        return;
+    if (i < sizeof(lcname)) {
+	/* short enough to be on the ignored-headers list */
+        lcname[i] = '\0';
+        if(strstr(ignored_headers, lcname))
+            return;
+    }
 
     res = g_list_prepend(res, libbalsa_create_hdr_pair(name, g_strdup(value)));
     *(GList **)user_data = res;
 }
 
-/* libbalsa_message_user_hdrs,
+/* 
  * libbalsa_message_user_hdrs_from_gmime:
  * 
  * returns allocated GList containing (header=>value) ALL headers
@@ -425,6 +431,7 @@ libbalsa_message_user_hdrs_from_gmime(GMimeMessage * message)
     */
     value = g_mime_message_get_header(message, "References");
     if (value) {
+#if BALSA_NEEDS_SEPARATE_USER_HEADERS
 	GMimeReferences *references, *reference;
 	reference = references = g_mime_references_decode(value);
 	while (reference) {
@@ -438,12 +445,21 @@ libbalsa_message_user_hdrs_from_gmime(GMimeMessage * message)
 	    reference = reference->next;
 	}
 	g_mime_references_clear(&references);
+#else
+        res = g_list_prepend(res,
+                             libbalsa_create_hdr_pair("References",
+                                                      g_strdup(value)));
+#endif
     }
 
     value = g_mime_message_get_header(message, "In-Reply-To");
     if (value) {
-	res = g_list_prepend(res, libbalsa_create_hdr_pair("In-Reply-To",
-				g_mime_utils_header_decode_text(value)));
+        res =
+            g_list_prepend(res,
+                           libbalsa_create_hdr_pair
+                           ("In-Reply-To",
+                            g_mime_utils_header_decode_text
+                            ((unsigned char *) value)));
     }
 
     g_mime_header_foreach(GMIME_OBJECT(message)->headers,
@@ -462,7 +478,7 @@ libbalsa_message_get_part_by_id(LibBalsaMessage* msg, const gchar* id)
     LibBalsaMessageBody* body = 
 	libbalsa_message_body_get_by_id(msg->body_list,	id);
     if(!body) return NULL;
-    return libbalsa_message_body_get_stream(body);
+    return libbalsa_message_body_get_stream(body, NULL); /*provide more info?*/
 }
 
 /* libbalsa_message_save:
@@ -481,11 +497,13 @@ libbalsa_message_save(LibBalsaMessage * message, const gchar *filename)
     if( (outfile = fopen(filename, "w")) == NULL) return FALSE;
     g_return_val_if_fail(outfile, FALSE);
 
-    msg_stream = libbalsa_mailbox_get_message_stream(message->mailbox, message);
+    msg_stream = libbalsa_message_stream(message);
     if (msg_stream == NULL)
 	return FALSE;
     out_stream = g_mime_stream_file_new(outfile);
+    libbalsa_mailbox_lock_store(message->mailbox);
     res = g_mime_stream_write_to_stream(msg_stream, out_stream);
+    libbalsa_mailbox_unlock_store(message->mailbox);
 
     g_object_unref(msg_stream);
     g_object_unref(out_stream);
@@ -803,15 +821,14 @@ libbalsa_message_append_part(LibBalsaMessage * message,
    address can be NULL.
 */
 void
-libbalsa_message_set_dispnotify(LibBalsaMessage *message, 
-				LibBalsaAddress *address)
+libbalsa_message_set_dispnotify(LibBalsaMessage * message,
+                                InternetAddress * ia)
 {
     g_return_if_fail(message);
-    if(message->headers->dispnotify_to) 
-	g_object_unref(message->headers->dispnotify_to);
-    message->headers->dispnotify_to = address;
-    if(address)
-	g_object_ref(message->headers->dispnotify_to);
+
+    internet_address_list_destroy(message->headers->dispnotify_to);
+    message->headers->dispnotify_to =
+        ia ? internet_address_list_prepend(NULL, ia) : NULL;
 }
 
 #ifndef MESSAGE_COPY_CONTENT
@@ -870,20 +887,21 @@ libbalsa_message_get_no(LibBalsaMessage* msg)
 
 #endif
 
-
-static LibBalsaAddress *
-libbalsa_address_new_from_gmime(const gchar *addr)
-{
-    LibBalsaAddress *address;
-
-    if (addr==NULL)
-	return NULL;
-    address = libbalsa_address_new_from_string(addr);
-    return address;
-}
-
 /* Populate headers from mime_msg, but only the members that are needed
  * all the time. */
+static InternetAddressList *
+lb_message_recipients(GMimeMessage * message, const gchar * type)
+{
+    const InternetAddressList *list;
+    InternetAddressList *copy = NULL;
+
+    for (list = g_mime_message_get_recipients(message, type);
+         list; list = list->next)
+        copy = internet_address_list_append(copy, list->address);
+
+    return copy;
+}
+
 static void
 lb_message_headers_basic_from_gmime(LibBalsaMessageHeaders *headers,
 				    GMimeMessage *mime_msg)
@@ -892,26 +910,14 @@ lb_message_headers_basic_from_gmime(LibBalsaMessageHeaders *headers,
     g_return_if_fail(mime_msg != NULL);
 
     if (!headers->from)
-        headers->from = libbalsa_address_new_from_gmime(mime_msg->from);
+        headers->from = internet_address_parse_string(mime_msg->from);
 
     if (!headers->date)
 	g_mime_message_get_date(mime_msg, &headers->date, NULL);
 
-    if (!headers->to_list) {
-	const InternetAddressList *addy, *start;
-	start = g_mime_message_get_recipients(mime_msg,
-					      GMIME_RECIPIENT_TYPE_TO);
-	for (addy = start; addy; addy = addy->next) {
-	    LibBalsaAddress *addr = addr = libbalsa_address_new();
-	    addr->full_name = g_strdup(addy->address->name);
-	    addr->address_list =
-		g_list_append(addr->address_list,
-			      g_strdup(addy->address->value.addr));
-	    if (addr)
-		headers->to_list = g_list_prepend(headers->to_list, addr);
-	}
-	headers->to_list = g_list_reverse(headers->to_list);
-    }
+    if (!headers->to_list)
+        headers->to_list =
+            lb_message_recipients(mime_msg, GMIME_RECIPIENT_TYPE_TO);
 
     if (!headers->content_type) {
 	/* If we could:
@@ -939,42 +945,22 @@ lb_message_headers_extra_from_gmime(LibBalsaMessageHeaders *headers,
     g_return_if_fail(mime_msg != NULL);
 
     if (!headers->reply_to)
-        headers->reply_to = libbalsa_address_new_from_gmime(mime_msg->reply_to);
+        headers->reply_to =
+	    internet_address_parse_string(mime_msg->reply_to);
 
     if (!headers->dispnotify_to)
-        headers->dispnotify_to = libbalsa_address_new_from_gmime(g_mime_message_get_header(mime_msg, "Disposition-Notification-To"));
+        headers->dispnotify_to =
+            internet_address_parse_string(g_mime_message_get_header
+                                          (mime_msg,
+                                           "Disposition-Notification-To"));
 
-    if (!headers->cc_list) {
-	const InternetAddressList *addy, *start;
-	start = g_mime_message_get_recipients(mime_msg,
-					      GMIME_RECIPIENT_TYPE_CC);
-	for (addy = start; addy; addy = addy->next) {
-	    LibBalsaAddress *addr = addr = libbalsa_address_new();
-	    addr->full_name = g_strdup(addy->address->name);
-	    addr->address_list =
-		g_list_append(addr->address_list,
-			      g_strdup(addy->address->value.addr));
-            headers->cc_list = g_list_prepend(headers->cc_list, addr);
-	}
-	headers->cc_list = g_list_reverse(headers->cc_list);
-    }
+    if (!headers->cc_list)
+        headers->cc_list =
+            lb_message_recipients(mime_msg, GMIME_RECIPIENT_TYPE_CC);
 
-    if (!headers->bcc_list) {
-	const InternetAddressList *addy, *start;
-	start = g_mime_message_get_recipients(mime_msg,
-					      GMIME_RECIPIENT_TYPE_BCC);
-	for (addy = start; addy; addy = addy->next) {
-	    LibBalsaAddress *addr = addr = libbalsa_address_new();
-	    addr->full_name = g_strdup(addy->address->name);
-	    addr->address_list =
-		g_list_append(addr->address_list,
-			      g_strdup(addy->address->value.addr));
-	    if (addr)
-		headers->bcc_list =
-		    g_list_prepend(headers->bcc_list, addr);
-	}
-	headers->bcc_list = g_list_reverse(headers->bcc_list);
-    }
+    if (!headers->bcc_list)
+        headers->bcc_list =
+            lb_message_recipients(mime_msg, GMIME_RECIPIENT_TYPE_BCC);
 
     /* Get fcc from message */
     if (!headers->fcc_url)
@@ -1005,7 +991,8 @@ libbalsa_message_init_from_gmime(LibBalsaMessage * message,
 #ifdef MESSAGE_COPY_CONTENT
     header = g_mime_message_get_subject(mime_msg);
     if (header) {
-	message->subj = g_mime_utils_header_decode_text(header);
+        message->subj =
+            g_mime_utils_header_decode_text((unsigned char *) header);
         canonize_header_value(message->subj);
 	libbalsa_utf8_sanitize(&message->subj, TRUE, NULL);
     }
@@ -1097,9 +1084,7 @@ void
 libbalsa_message_set_in_reply_to_from_string(LibBalsaMessage * message,
 					     const gchar * str)
 {
-    g_return_if_fail(message->in_reply_to == NULL);
-
-    if (str) {
+    if (!message->in_reply_to && str) {
 	/* FIXME for Balsa's old non-compliant header */
 	gchar *p = strrchr(str, ';');
 	p = p ? g_strndup(str, p - str) : g_strdup(str);
@@ -1125,12 +1110,33 @@ static gboolean
 lbmsg_set_header(LibBalsaMessage *message, const gchar *name,
                  const gchar* value, gboolean all)
 {
+    gchar *val = NULL;
+
+    if (libbalsa_text_attr_string(value)) {
+        /* Broken header: force it to utf8 using Balsa's fallback
+         * charset, then rfc2047-encode it for passing to the
+         * appropriate GMime decoder. */
+        gchar *tmp = g_strdup(value);
+        libbalsa_utf8_sanitize(&tmp, TRUE, NULL);
+        val = g_mime_utils_header_encode_text((unsigned char *) tmp);
+        g_free(tmp);
+#ifdef DEBUG
+        g_print("%s: non-ascii \"%s\" header \"%s\" encoded as \"%s\"\n",
+                __func__, name, value, val);
+#endif /* DEBUG */
+        value = val;
+    }
+
     if (g_ascii_strcasecmp(name, "Subject") == 0) {
-	if (!strcmp(value, "DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA"))
+	if (!strcmp(value,
+                    "DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA")) {
+            g_free(val);
 	    return FALSE;
+        }
 #if MESSAGE_COPY_CONTENT
 	g_free(message->subj);
-        message->subj = g_mime_utils_header_decode_text(value);
+        message->subj =
+	    g_mime_utils_header_decode_text((unsigned char *) value);
 	libbalsa_utf8_sanitize(&message->subj, TRUE, NULL);
 #endif
     } else
@@ -1138,11 +1144,10 @@ lbmsg_set_header(LibBalsaMessage *message, const gchar *name,
 	message->headers->date = g_mime_utils_header_decode_date(value, NULL);
     } else
     if (g_ascii_strcasecmp(name, "From") == 0) {
-        message->headers->from = libbalsa_address_new_from_string(value);
+        message->headers->from = internet_address_parse_string(value);
     } else
     if (g_ascii_strcasecmp(name, "To") == 0) {
-	message->headers->to_list =
-	    libbalsa_address_new_list_from_string(value);
+	message->headers->to_list = internet_address_parse_string(value);
     } else
     if (g_ascii_strcasecmp(name, "In-Reply-To") == 0) {
 	libbalsa_message_set_in_reply_to_from_string(message, value);
@@ -1163,9 +1168,12 @@ lbmsg_set_header(LibBalsaMessage *message, const gchar *name,
     } else
 #endif
     if (all)
-	message->headers->user_hdrs =
-	    g_list_append(message->headers->user_hdrs,
-			  libbalsa_create_hdr_pair(name, g_strdup(value)));
+        message->headers->user_hdrs =
+            g_list_prepend(message->headers->user_hdrs,
+                           libbalsa_create_hdr_pair(name,
+                                                    g_strdup(value)));
+
+    g_free(val);
 
     return TRUE;
 }
@@ -1206,28 +1214,25 @@ libbalsa_message_set_headers_from_string(LibBalsaMessage *message,
 }
 
 void
-libbalsa_message_load_envelope(LibBalsaMessage *message)
+libbalsa_message_load_envelope_from_stream(LibBalsaMessage * message,
+                                           GMimeStream *gmime_stream)
 {
-    GMimeStream *gmime_stream;
     GMimeStream *gmime_stream_buffer;
     GByteArray *line;
-    char lookahead;
+    guchar lookahead;
     gboolean ret = FALSE;
 
-    gmime_stream =
-	libbalsa_mailbox_get_message_stream(message->mailbox, message);
-    if (!gmime_stream)
-	return;
-    gmime_stream_buffer = g_mime_stream_buffer_new(gmime_stream,
-					GMIME_STREAM_BUFFER_BLOCK_READ);
-    g_object_unref(gmime_stream);
+    libbalsa_mime_stream_shared_lock(gmime_stream);
+    gmime_stream_buffer =
+        g_mime_stream_buffer_new(gmime_stream,
+                                 GMIME_STREAM_BUFFER_BLOCK_READ);
 
     line = g_byte_array_new();
     do {
 	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
 	while (!g_mime_stream_eos(gmime_stream_buffer)
-	       && g_mime_stream_read(gmime_stream_buffer, &lookahead, 1) == 1)
-	{
+	       && g_mime_stream_read(gmime_stream_buffer,
+		                     (char *) &lookahead, 1) == 1) {
 		if (lookahead == ' ' || lookahead == '\t') {
 		    g_byte_array_append(line, &lookahead, 1);
 		    g_mime_stream_buffer_readln(gmime_stream_buffer, line);
@@ -1240,7 +1245,9 @@ libbalsa_message_load_envelope(LibBalsaMessage *message)
 	    break;
 	}
 	line->data[line->len-1]='\0'; /* terminate line by overwriting '\n' */
-	if (!lb_message_set_headers_from_string(message, line->data, FALSE)) {
+	if (!lb_message_set_headers_from_string(message,
+				                (gchar *) line->data,
+						FALSE)) {
 	    /* Ignore error return caused by malformed header. */
 	}
 	if (lookahead == '\n') {/* end of headers */
@@ -1250,9 +1257,75 @@ libbalsa_message_load_envelope(LibBalsaMessage *message)
 	line->len = 0;
 	g_byte_array_append(line, &lookahead, 1);
     } while (!g_mime_stream_eos(gmime_stream_buffer));
-    if (ret) {
-	/* calculate size */
-    }
+    if (ret)
+        message->length = g_mime_stream_length(gmime_stream);
     g_object_unref(gmime_stream_buffer);
+    g_mime_stream_reset(gmime_stream);
+    libbalsa_mime_stream_shared_unlock(gmime_stream);
     g_byte_array_free(line, TRUE);
 }
+
+void
+libbalsa_message_load_envelope(LibBalsaMessage *message)
+{
+    GMimeStream *gmime_stream;
+
+    gmime_stream = libbalsa_message_stream(message);
+    if (!gmime_stream)
+	return;
+
+    libbalsa_message_load_envelope_from_stream(message, gmime_stream);
+    g_object_unref(gmime_stream);
+}
+
+GMimeStream *
+libbalsa_message_stream(LibBalsaMessage * message)
+{
+    LibBalsaMailbox *mailbox;
+    GMimeStream *mime_stream;
+
+    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), NULL);
+    mailbox = message->mailbox;
+    g_return_val_if_fail(mailbox != NULL || message->mime_msg != NULL,
+                         NULL);
+
+    if (mailbox)
+        return libbalsa_mailbox_get_message_stream(mailbox,
+                                                   message->msgno);
+
+    mime_stream = g_mime_stream_mem_new();
+    g_mime_object_write_to_stream(GMIME_OBJECT(message->mime_msg),
+                                  mime_stream);
+    g_mime_stream_reset(mime_stream);
+
+    return mime_stream;
+}
+
+gboolean
+libbalsa_message_copy(LibBalsaMessage * message, LibBalsaMailbox * dest,
+                      GError ** err)
+{
+    LibBalsaMailbox *mailbox;
+    gboolean retval;
+
+    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), FALSE);
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX(dest), FALSE);
+    mailbox = message->mailbox;
+    g_return_val_if_fail(mailbox != NULL || message->mime_msg != NULL,
+                         FALSE);
+
+    if (mailbox) {
+        GArray *msgnos = g_array_sized_new(FALSE, FALSE, sizeof(guint), 1);
+        g_array_append_val(msgnos, message->msgno);
+        retval =
+            libbalsa_mailbox_messages_copy(mailbox, msgnos, dest, err);
+        g_array_free(msgnos, TRUE);
+    } else {
+        GMimeStream *mime_stream = libbalsa_message_stream(message);
+        retval = libbalsa_mailbox_add_message(dest, mime_stream,
+                                              message->flags, err);
+        g_object_unref(mime_stream);
+    }
+
+    return retval;
+}
