@@ -878,7 +878,7 @@ disp_recipient_status(smtp_recipient_t recipient,
 
   if(status->code != 0 && status->code != 250) {
       libbalsa_information(
-                           LIBBALSA_INFORMATION_WARNING, 
+                           LIBBALSA_INFORMATION_ERROR,
                            _("Could not send the message to %s:\n"
                              "%d: %s\n"
                              "Message left in your outbox.\n"), 
@@ -936,10 +936,9 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
                              LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
 	}
     } else {
-	/* XXX - Show the poor user the status codes and message. */
+	/* Check whether it was a problem with transfer. */
         if(*(gboolean*)be_verbose) {
             int cnt = 0;
-            /* status = smtp_reverse_path_status(message); */
             if(status->code != 250 && status->code != 0) {
                 libbalsa_information(
                                      LIBBALSA_INFORMATION_WARNING,
@@ -950,12 +949,21 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
                 cnt++;
             }
             smtp_enumerate_recipients (message, disp_recipient_status, &cnt);
-            if(cnt==0) { /* other error */
-            libbalsa_information(
-           LIBBALSA_INFORMATION_WARNING,
-           _("Message submission problem, placing it into your outbox.\n"
-             "System will attempt to resubmit the message until you delete it."));
-
+            if(cnt==0) { /* other error, maybe sender or message size? */
+                status = smtp_reverse_path_status(message);
+                if(status->code != 250 && status->code != 0) {
+                    libbalsa_information
+                        (LIBBALSA_INFORMATION_WARNING,
+                         _("Relaying refused:\n"
+                           "%d: %s\n"
+                           "Message left in your outbox.\n"), 
+                         status->code, status->text);
+                } else
+                    libbalsa_information
+                        (LIBBALSA_INFORMATION_ERROR,
+                         _("Message submission problem, placing it into your outbox.\n"
+                           "System will attempt to resubmit the message until you delete it."));
+                
             }
         }
     }
