@@ -25,33 +25,18 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
 
 #include "misc.h"
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 #include "misc.h"
+#include "i18n.h"
 
-
-
-#if GTK_CHECK_VERSION(2, 4, 0)
-# define USE_GTK_ACTION TRUE
-#endif
-
-#if USE_GTK_ACTION
 static void close_cb(GtkAction * action, gpointer data);
 static void copy_cb(GtkAction * action, gpointer data);
 static void select_all_cb(GtkAction * action, gpointer data);
 static void lsv_escape_cb(GtkAction * action, gpointer data);
-#else
-static void close_cb(GtkWidget* w, gpointer data);
-static void copy_cb(GtkWidget * w, gpointer data);
-static void select_all_cb(GtkWidget* w, gpointer data);
-static void lsv_escape_cb(GtkWidget * widget, gpointer data);
-#endif /* USE_GTK_ACTION */
 
-#if USE_GTK_ACTION
 /* Normal items */
 static GtkActionEntry entries[] = {
     /* Top level */
@@ -90,49 +75,9 @@ static const char *ui_description =
 "    </menu>"
 "  </menubar>"
 "</ui>";
-#else
-static GnomeUIInfo file_menu[] = {
-#define MENU_FILE_INCLUDE_POS 1
-    GNOMEUIINFO_MENU_CLOSE_ITEM(close_cb, NULL),
-    GNOMEUIINFO_END
-};
-
-static GnomeUIInfo edit_menu[] = {
-#define MENU_EDIT_INCLUDE_POS 0
-    GNOMEUIINFO_MENU_COPY_ITEM(copy_cb, NULL),
-    GNOMEUIINFO_SEPARATOR,
-    {GNOME_APP_UI_ITEM, N_("_Select Text"),
-     N_("Select entire mail"),
-     select_all_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE,
-     NULL, 'A', GDK_CONTROL_MASK, NULL
-    },
-    GNOMEUIINFO_END
-};
-
-static GnomeUIInfo view_menu[] = {
-#define MENU_VIEW_ESCAPE_POS 0
-    GNOMEUIINFO_TOGGLEITEM(N_("_Escape Special Characters"),
-                           N_("Escape special and non-ASCII characters"),
-                           lsv_escape_cb, NULL),
-    GNOMEUIINFO_END
-};
-
-static GnomeUIInfo main_menu[] = {
-#define SOURCE_FILE_MENU 2
-    GNOMEUIINFO_MENU_FILE_TREE(file_menu),
-    GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
-    GNOMEUIINFO_MENU_VIEW_TREE(view_menu),
-    GNOMEUIINFO_END
-};
-#endif /* USE_GTK_ACTION */
-
 
 static void 
-#if USE_GTK_ACTION
 select_all_cb(GtkAction * action, gpointer data)
-#else
-select_all_cb(GtkWidget * w, gpointer data)
-#endif /* USE_GTK_ACTION */
 {
     GtkTextView *text = g_object_get_data(G_OBJECT(data), "text");
     GtkTextBuffer *buffer = gtk_text_view_get_buffer(text);
@@ -144,11 +89,7 @@ select_all_cb(GtkWidget * w, gpointer data)
 }
 
 static void
-#if USE_GTK_ACTION
 copy_cb(GtkAction * action, gpointer data)
-#else
-copy_cb(GtkWidget * w, gpointer data)
-#endif /* USE_GTK_ACTION */
 {
     GtkTextView *text = g_object_get_data(G_OBJECT(data), "text");
     GtkTextBuffer *buffer = gtk_text_view_get_buffer(text);
@@ -158,11 +99,7 @@ copy_cb(GtkWidget * w, gpointer data)
 }
 
 static void
-#if USE_GTK_ACTION
 close_cb(GtkAction * action, gpointer data)
-#else
-close_cb(GtkWidget* w, gpointer data)
-#endif /* USE_GTK_ACTION */
 {
     gtk_widget_destroy(GTK_WIDGET(data));
 }
@@ -203,11 +140,7 @@ lsv_show_message(const char *message, LibBalsaSourceViewerInfo * lsvi,
 }
 
 static void
-#if USE_GTK_ACTION
 lsv_escape_cb(GtkAction * action, gpointer data)
-#else
-lsv_escape_cb(GtkWidget * widget, gpointer data)
-#endif /* USE_GTK_ACTION */
 {
     LibBalsaSourceViewerInfo *lsvi =
         g_object_get_data(G_OBJECT(data), "lsvi");
@@ -221,25 +154,23 @@ lsv_escape_cb(GtkWidget * widget, gpointer data)
 			     _("Mailbox closed"));
 	return;
     }
-    msg_stream = libbalsa_mailbox_get_message_stream(msg->mailbox, msg);
+    msg_stream = libbalsa_mailbox_get_message_stream(msg->mailbox, msg->msgno);
     if (msg_stream == NULL)
 	return;
 
     mem_stream = g_mime_stream_mem_new();
+    libbalsa_mailbox_lock_store(msg->mailbox);
     g_mime_stream_write_to_stream(msg_stream, mem_stream);
+    libbalsa_mailbox_unlock_store(msg->mailbox);
     g_mime_stream_write(mem_stream, "", 1); /* close string */
-    raw_message = GMIME_STREAM_MEM(mem_stream)->buffer->data;
+    raw_message = (char *) GMIME_STREAM_MEM(mem_stream)->buffer->data;
 
-#if USE_GTK_ACTION
     *(lsvi->escape_specials) =
 	gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
-#else
-    *(lsvi->escape_specials) = GTK_CHECK_MENU_ITEM(widget)->active;
-#endif /* USE_GTK_ACTION */
     lsv_show_message(raw_message, lsvi, *(lsvi->escape_specials));
 
-    g_mime_stream_unref(msg_stream);
-    g_mime_stream_unref(mem_stream);
+    g_object_unref(msg_stream);
+    g_object_unref(mem_stream);
 }
 
 static void
@@ -253,9 +184,8 @@ lsv_window_destroy_notify(LibBalsaSourceViewerInfo * lsvi)
    pops up a window containing the source of the message msg.
 */
 
-#if USE_GTK_ACTION
-static void
-lbsv_app_set_menus(GnomeApp * app, GtkAction ** action)
+static GtkWidget*
+lbsv_app_set_menus(GtkWindow * app, GtkAction ** action)
 {
     GtkWidget *window;
     GtkWidget *menubar;
@@ -272,11 +202,6 @@ lbsv_app_set_menus(GnomeApp * app, GtkAction ** action)
     gtk_action_group_add_toggle_actions(action_group, toggle_entries,
                                         G_N_ELEMENTS(toggle_entries),
                                         window);
-#if WE_HAVE_RADIO_ACTIONS
-    gtk_action_group_add_radio_actions(action_group, radio_entries,
-                                       G_N_ELEMENTS(radio_entries), 0,
-                                       radio_action_callback, window);
-#endif                          /* WE_HAVE_RADIO_ACTIONS */
 
     ui_manager = gtk_ui_manager_new();
     gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);
@@ -289,16 +214,15 @@ lbsv_app_set_menus(GnomeApp * app, GtkAction ** action)
                                            -1, &error)) {
         g_message("building menus failed: %s", error->message);
         g_error_free(error);
-        return;
+        return NULL;
     }
 
     menubar = gtk_ui_manager_get_widget(ui_manager, "/MainMenu");
-    gnome_app_set_menus(app, GTK_MENU_BAR(menubar));
 
     *action =
         gtk_ui_manager_get_action(ui_manager, "/MainMenu/ViewMenu/Escape");
+    return menubar;
 }
-#endif /* USE_GTK_ACTION */
 
 void
 libbalsa_show_message_source(LibBalsaMessage* msg, const gchar * font,
@@ -306,11 +230,9 @@ libbalsa_show_message_source(LibBalsaMessage* msg, const gchar * font,
 {
     GtkWidget *text;
     PangoFontDescription *desc;
-    GtkWidget *interior;
+    GtkWidget *vbox, *interior;
     GtkWidget *window;
-#if USE_GTK_ACTION
     GtkAction *escape_action = NULL;
-#endif /* USE_GTK_ACTION */
     LibBalsaSourceViewerInfo *lsvi;
 
     g_return_if_fail(msg);
@@ -323,11 +245,7 @@ libbalsa_show_message_source(LibBalsaMessage* msg, const gchar * font,
     pango_font_description_free(desc);
 
     gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
-#if GTK_CHECK_VERSION(2, 4, 0)
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
-#else
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
-#endif
 
     interior = gtk_scrolled_window_new(GTK_TEXT_VIEW(text)->hadjustment,
                                        GTK_TEXT_VIEW(text)->vadjustment);
@@ -335,16 +253,17 @@ libbalsa_show_message_source(LibBalsaMessage* msg, const gchar * font,
                                    GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
     gtk_container_add(GTK_CONTAINER(interior), GTK_WIDGET(text));
 
-    window = gnome_app_new("balsa", _("Message Source"));
+    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_title(GTK_WINDOW(window), _("Message Source"));
     g_object_set_data(G_OBJECT(window), "text", text);
-#if USE_GTK_ACTION
-    lbsv_app_set_menus(GNOME_APP(window), &escape_action);
-#else
-    gnome_app_create_menus_with_data(GNOME_APP(window), main_menu, window);
-#endif /* USE_GTK_ACTION */
     gtk_window_set_wmclass(GTK_WINDOW(window), "message-source", "Balsa");
     gtk_window_set_default_size(GTK_WINDOW(window), 500, 400);
-    gnome_app_set_contents(GNOME_APP(window), interior);
+    vbox = gtk_vbox_new(FALSE, 1);
+    gtk_box_pack_start(GTK_BOX(vbox), 
+                       lbsv_app_set_menus(GTK_WINDOW(window),
+                                          &escape_action), FALSE, TRUE, 1);
+    gtk_box_pack_start_defaults(GTK_BOX(vbox), interior);
+    gtk_container_add(GTK_CONTAINER(window), vbox);
 
     lsvi = g_new(LibBalsaSourceViewerInfo, 1);
     lsvi->msg = msg;
@@ -356,44 +275,9 @@ libbalsa_show_message_source(LibBalsaMessage* msg, const gchar * font,
                            (GDestroyNotify) lsv_window_destroy_notify);
 
     gtk_widget_show_all(window);
-#if USE_GTK_ACTION
     if (*escape_specials)
         gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(escape_action),
                                      TRUE);
     else
         lsv_escape_cb(escape_action, window);
-#else
-    if(*escape_specials) 
-        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
-                                       (view_menu[MENU_VIEW_ESCAPE_POS].
-                                        widget), *escape_specials);
-    else
-        lsv_escape_cb(view_menu[MENU_VIEW_ESCAPE_POS].widget, window);
-#endif /* USE_GTK_ACTION */
 }
-
-
-#if 0
-/* testing program */
-int
-main(int argc, char* argv[])
-{
-    int i, shown = 0;
-    gtk_init(&argc, &argv);
-
-    for(i=1; i<argc; i++) {
-	FILE* f = fopen(argv[i], "r");
-	if(f) {
-	    show_file(f);
-	    fclose(f);
-	    shown = 1;
-	}
-    }
-    if(shown)
-	gtk_main();
-    else
-	fprintf(stderr, "No sensible args passed.\n");
-
-    return !shown;
-}
-#endif
