@@ -570,7 +570,14 @@ bmbl_tree_expand(GtkTreeView * tree_view, GtkTreeIter * iter,
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN, pixbuf,
                            -1);
-        g_object_unref(pixbuf);
+        if(pixbuf) g_object_unref(pixbuf);
+        else { 
+            static int p=1;
+            if(p){
+                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_OPEN);
+                p = 0;
+            }
+        }
 #endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     }
     g_object_unref(mbnode);
@@ -654,7 +661,14 @@ bmbl_tree_collapse(GtkTreeView * tree_view, GtkTreeIter * iter,
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN, pixbuf,
                            -1);
-        g_object_unref(pixbuf);
+        if(pixbuf) g_object_unref(pixbuf);
+        else { 
+            static int p=1;
+            if(p){
+                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_CLOSED);
+                p = 0;
+            }
+        }
 #endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     }
     g_object_unref(mbnode);
@@ -1011,24 +1025,28 @@ update_mailbox_idle(struct update_mbox_data *umd)
         g_object_set_data(G_OBJECT(umd->mailbox), "mblist-update", NULL);
 
         if (balsa_app.mblist_tree_store) {
+            gboolean subscribed =
+                libbalsa_mailbox_get_subscribe(umd->mailbox) !=
+                LB_MAILBOX_SUBSCRIBE_NO;
             bmbl_update_mailbox(balsa_app.mblist_tree_store, umd->mailbox);
-            check_new_messages_count(umd->mailbox, umd->notify);
+            check_new_messages_count(umd->mailbox, umd->notify
+                                     && subscribed);
 
-            if (libbalsa_mailbox_get_subscribe(umd->mailbox) !=
-                LB_MAILBOX_SUBSCRIBE_NO
-                && libbalsa_mailbox_get_unread(umd->mailbox) > 0)
-                g_signal_emit(balsa_app.mblist,
-                              balsa_mblist_signals[HAS_UNREAD_MAILBOX], 0,
-                              TRUE);
-            else {
-                GList *unread_mailboxes =
-                    balsa_mblist_find_all_unread_mboxes(NULL);
-                if (unread_mailboxes)
-                    g_list_free(unread_mailboxes);
-                else
+            if (subscribed) {
+                if (libbalsa_mailbox_get_unread(umd->mailbox) > 0)
                     g_signal_emit(balsa_app.mblist,
                                   balsa_mblist_signals[HAS_UNREAD_MAILBOX],
-                                  0, FALSE);
+                                  0, TRUE);
+                else {
+                    GList *unread_mailboxes =
+                        balsa_mblist_find_all_unread_mboxes(NULL);
+                    if (unread_mailboxes)
+                        g_list_free(unread_mailboxes);
+                    else
+                        g_signal_emit(balsa_app.mblist,
+                                      balsa_mblist_signals
+                                      [HAS_UNREAD_MAILBOX], 0, FALSE);
+                }
             }
         }
     }
@@ -1217,7 +1235,7 @@ get_lru_descendant(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
        (!dt->mbnode || (mbnode->last_use < dt->mbnode->last_use)) )
     {
         if (dt->mbnode)
-            g_object_unref(mbnode);
+            g_object_unref(dt->mbnode);
         dt->mbnode = mbnode; 
     }
 
@@ -1245,7 +1263,9 @@ balsa_mblist_close_lru_peer_mbx(BalsaMBList * mblist,
     dt.mbnode = NULL;
     gtk_tree_model_foreach(model, get_lru_descendant, &dt);
     if(dt.mbnode) {
+        gdk_threads_enter();
         balsa_window_close_mbnode(balsa_app.main_window, dt.mbnode);
+        gdk_threads_leave();
         g_object_unref(dt.mbnode);
     }
     return dt.mbnode != NULL;
@@ -1394,6 +1414,7 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
                        TOTAL_COLUMN,  "",
                        -1);
 #else                           /* GTK_CHECK_VERSION(2, 8, 0) */
+    /* render icon may fail if the installation was incomplete */
     pixbuf = gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
                                     icon, GTK_ICON_SIZE_MENU, NULL);
     gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
@@ -1405,7 +1426,11 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
                        UNREAD_COLUMN, "",
                        TOTAL_COLUMN,  "",
                        -1);
-    g_object_unref(pixbuf);
+    if(pixbuf) g_object_unref(pixbuf);
+    else { 
+        static int p=1;
+        if(p){ printf("icon %s not found.\n", icon); p = 0; }
+    }
 #endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     g_free(tmp);
 
@@ -1541,7 +1566,11 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
                            NAME_COLUMN, name,
                            WEIGHT_COLUMN, weight,
                            -1);
-        g_object_unref(pixbuf);
+        if(pixbuf) g_object_unref(pixbuf);
+        else { 
+            static int p=1;
+            if(p){ printf("icon %s not found.\n", icon); p = 0; }
+        }
 #endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
         g_free(tmp);
 
@@ -2324,6 +2353,20 @@ balsa_mblist_mailbox_node_append(BalsaMailboxNode * root,
     gtk_tree_store_append(balsa_app.mblist_tree_store, &iter, parent_iter);
     bmbl_store_redraw_mbnode(&iter, mbnode);
 
+    if (parent_iter) {
+        /* Check whether this node is exposed. */
+        GtkTreePath *parent_path =
+            gtk_tree_model_get_path(model, parent_iter);
+        if (gtk_tree_view_row_expanded(GTK_TREE_VIEW(balsa_app.mblist),
+                                       parent_path)
+            && !mbnode->scanned) {
+            /* Check this node for children. */
+            balsa_mailbox_node_append_subtree(mbnode);
+            mbnode->scanned = TRUE;
+        }
+        gtk_tree_path_free(parent_path);
+    }
+
     /* The tree-store owns mbnode. */
     g_object_unref(mbnode);
 
