@@ -26,12 +26,15 @@
 #include "config.h"
 
 #include <string.h>
+
 #include "mailbox-filter.h"
 #include "filter-funcs.h"
 #include "filter-run.h"
 #include "balsa-app.h"
 #include "save-restore.h"
 
+#include "i18n.h"	/* Must come after balsa-app.h. */
+
 /* Global vars */
 
 extern GList * fr_dialogs_opened;
@@ -80,25 +83,19 @@ build_selected_filters_list(GtkTreeView * filter_list, gboolean to_run)
 
 /*
  * Run the selected filters on the mailbox
- * Returns TRUE if OK, or FALSE if errors occured
+ * Returns TRUE if OK, or FALSE if errors occurred
  */
 static gboolean
-run_filters_on_mailbox(GtkTreeView * filter_list, LibBalsaMailbox * mbox)
+run_filters_on_mailbox(GSList * filters, LibBalsaMailbox * mbox)
 {
-    GSList *filters = build_selected_filters_list(filter_list, TRUE);
-    GSList *lst;
     guint sent_to_trash;
 
-    if (!filters)
-	return TRUE;
-    if (!filters_prepare_to_run(filters)) {
-	g_slist_free(filters);
+    if (!filters_prepare_to_run(filters))
 	return FALSE;
-    }
 
     sent_to_trash = 0;
-    for (lst = filters; lst; lst = g_slist_next(lst)) {
-	LibBalsaFilter *filter = lst->data;
+    for (; filters; filters = filters->next) {
+	LibBalsaFilter *filter = filters->data;
 	LibBalsaMailboxSearchIter *search_iter;
 	guint msgno;
 	GArray *messages;
@@ -130,7 +127,6 @@ run_filters_on_mailbox(GtkTreeView * filter_list, LibBalsaMailbox * mbox)
 	libbalsa_mailbox_unregister_msgnos(mbox, messages);
 	g_array_free(messages, TRUE);
     }
-    g_slist_free(filters);
 
     if (sent_to_trash)
 	enable_empty_trash(balsa_app.main_window, TRASH_FULL);
@@ -167,12 +163,6 @@ void fr_dialog_response(GtkWidget * widget, gint response,
 
     p=BALSA_FILTER_RUN_DIALOG(widget);
     switch (response) {
-    case GTK_RESPONSE_APPLY:    /* Apply button */
-	if (!run_filters_on_mailbox(p->selected_filters,p->mbox))
-	    balsa_information_parented(GTK_WINDOW(widget),
-		    LIBBALSA_INFORMATION_ERROR,
-		    _("Error when applying filters"));
-	return;
     case GTK_RESPONSE_OK:       /* OK button */
 	save_filters(p);
 	break;
@@ -202,10 +192,70 @@ void fr_dialog_response(GtkWidget * widget, gint response,
     gtk_widget_destroy(GTK_WIDGET(p));
 }
 
+
 /* 
- *Callbacks for left/right buttons
+ *Callbacks for apply/left/right buttons
  */
 
+static void
+fr_apply_selected_pressed_func(GtkTreeModel * model, GtkTreePath * path,
+                               GtkTreeIter * iter, gpointer data)
+{
+    LibBalsaFilter *fil;
+    BalsaFilterRunDialog *p = data;
+
+    gtk_tree_model_get(model, iter, DATA_COLUMN, &fil, -1);
+    /* Check for move to self: */
+    if (fil->action == FILTER_RUN || fil->action == FILTER_TRASH ||
+        strcmp(fil->action_string, p->mbox->url) != 0)
+        p->filters = g_slist_append(p->filters, fil);
+}
+
+void
+fr_apply_selected_pressed(BalsaFilterRunDialog * p)
+{
+    GtkTreeSelection *selection =
+        gtk_tree_view_get_selection(p->available_filters);
+
+    p->filters = NULL;
+    gtk_tree_selection_selected_foreach(selection,
+                                        fr_apply_selected_pressed_func, p);
+
+    if (!p->filters)
+        return;
+
+    if (!run_filters_on_mailbox(p->filters, p->mbox))
+        balsa_information_parented(GTK_WINDOW(p),
+                                   LIBBALSA_INFORMATION_ERROR,
+                                   _("Error when applying filters"));
+    else
+        balsa_information(LIBBALSA_INFORMATION_MESSAGE,
+                          _("Filter applied to \"%s\"."),
+                          p->mbox->name);
+
+    g_slist_free(p->filters);
+}
+
+void
+fr_apply_now_pressed(BalsaFilterRunDialog* p)
+{
+    GSList *filters;
+
+    filters = build_selected_filters_list(p->selected_filters, TRUE);
+
+    if (!filters)
+        return;
+
+    if (!run_filters_on_mailbox(filters, p->mbox))
+        balsa_information_parented(GTK_WINDOW(p),
+                                   LIBBALSA_INFORMATION_ERROR,
+                                   _("Error when applying filters"));
+    else
+        balsa_information(LIBBALSA_INFORMATION_MESSAGE,
+                          _("Filter applied to \"%s\"."),
+                          p->mbox->name);
+}
+
 static void
 fr_add_pressed_func(GtkTreeModel * model, GtkTreePath * path,
                     GtkTreeIter * iter, gpointer data)
@@ -225,6 +275,9 @@ fr_add_pressed_func(GtkTreeModel * model, GtkTreePath * path,
         GtkTreeIter sel_iter;
 
         mf->actual_filter = fil;
+#if DEBUG
+        printf("fr_add_presed adds fil=%p\n", fil);
+#endif
         mf->when = FILTER_WHEN_NEVER;
         gtk_list_store_append(GTK_LIST_STORE(sel_model), &sel_iter);
         gtk_list_store_set(GTK_LIST_STORE(sel_model), &sel_iter,
@@ -408,59 +461,62 @@ selected_list_activated(GtkTreeView * treeview, GtkTreePath * path,
  */
 
 /*
- * fe_down_pressed()
- *
- * Callback for the "Down" button
- */ 
+ * helper
+ */
 static void
-fr_pressed_func(GtkTreeModel * model, GtkTreePath * path,
-                GtkTreeIter * iter, gpointer data)
+fr_swap(GtkTreeModel * model, GtkTreeIter * iter1, GtkTreeIter * iter2,
+        GtkTreeSelection * selection, GtkTreePath * path,
+        BalsaFilterRunDialog * p)
 {
-    GtkTreeIter *new_iter = data;
-    *new_iter = *iter;
-}
-
-void
-fr_down_pressed(GtkWidget * widget, gpointer data)
-{
-    BalsaFilterRunDialog *p = data;
-    GtkTreeModel *model =
-        gtk_tree_view_get_model(p->selected_filters);
-    GtkTreeSelection *selection =
-        gtk_tree_view_get_selection(p->selected_filters);
-    GtkTreeIter iter1, iter2;
     LibBalsaMailboxFilter *fil1, *fil2;
     gboolean incoming1, closing1;
     gboolean incoming2, closing2;
 
-    iter1.stamp = 0;
-    gtk_tree_selection_selected_foreach(selection, fr_pressed_func, &iter1);
-    if (iter1.stamp) {
+    gtk_tree_model_get(model, iter1,
+                       DATA_COLUMN, &fil1,
+                       INCOMING_COLUMN, &incoming1,
+                       CLOSING_COLUMN, &closing1, -1);
+    gtk_tree_model_get(model, iter2,
+                       DATA_COLUMN, &fil2,
+                       INCOMING_COLUMN, &incoming2,
+                       CLOSING_COLUMN, &closing2, -1);
+    gtk_list_store_set(GTK_LIST_STORE(model), iter1,
+                       NAME_COLUMN, fil2->actual_filter->name,
+                       DATA_COLUMN, fil2,
+                       INCOMING_COLUMN, incoming2,
+                       CLOSING_COLUMN, closing2, -1);
+    gtk_list_store_set(GTK_LIST_STORE(model), iter2,
+                       NAME_COLUMN, fil1->actual_filter->name,
+                       DATA_COLUMN, fil1,
+                       INCOMING_COLUMN, incoming1,
+                       CLOSING_COLUMN, closing1, -1);
+
+    gtk_tree_selection_select_path(selection, path);
+    gtk_tree_view_scroll_to_cell(p->selected_filters, path, NULL,
+                                 FALSE, 0, 0);
+    p->filters_modified = TRUE;
+}
+
+/*
+ * fr_down_pressed()
+ *
+ * Callback for the "Down" button
+ */
+void
+fr_down_pressed(GtkWidget * widget, gpointer data)
+{
+    BalsaFilterRunDialog *p = data;
+    GtkTreeModel *model;
+    GtkTreeSelection *selection =
+        gtk_tree_view_get_selection(p->selected_filters);
+    GtkTreeIter iter1, iter2;
+
+    if (gtk_tree_selection_get_selected(selection, &model, &iter1)) {
         iter2 = iter1;
         if (gtk_tree_model_iter_next(model, &iter2)) {
-            gtk_tree_model_get(model, &iter1,
-                               DATA_COLUMN, &fil1,
-                               INCOMING_COLUMN, &incoming1,
-                               CLOSING_COLUMN, &closing1,
-                               -1);
-            gtk_tree_model_get(model, &iter2,
-                               DATA_COLUMN, &fil2,
-                               INCOMING_COLUMN, &incoming2,
-                               CLOSING_COLUMN, &closing2,
-                               -1);
-            gtk_list_store_set(GTK_LIST_STORE(model), &iter1,
-                               NAME_COLUMN, fil2->actual_filter->name,
-                               DATA_COLUMN, fil2,
-                               INCOMING_COLUMN, incoming2,
-                               CLOSING_COLUMN, closing2,
-                               -1);
-            gtk_list_store_set(GTK_LIST_STORE(model), &iter2,
-                               NAME_COLUMN, fil1->actual_filter->name,
-                               DATA_COLUMN, fil1,
-                               INCOMING_COLUMN, incoming1,
-                               CLOSING_COLUMN, closing1,
-                               -1);
-            p->filters_modified = TRUE;
+            GtkTreePath *path = gtk_tree_model_get_path(model, &iter2);
+            fr_swap(model, &iter1, &iter2, selection, path, p);
+            gtk_tree_path_free(path);
         }
     }
 }                               /* end fe_down_pressed */
@@ -474,45 +530,16 @@ void
 fr_up_pressed(GtkWidget * widget, gpointer data)
 {
     BalsaFilterRunDialog *p = data;
-    GtkTreeModel *model =
-        gtk_tree_view_get_model(p->selected_filters);
+    GtkTreeModel *model;
     GtkTreeSelection *selection =
         gtk_tree_view_get_selection(p->selected_filters);
     GtkTreeIter iter1, iter2;
-    LibBalsaMailboxFilter *fil1, *fil2;
-    gboolean incoming1, closing1;
-    gboolean incoming2, closing2;
 
-    iter1.stamp = 0;
-    gtk_tree_selection_selected_foreach(selection, fr_pressed_func, &iter1);
-    if (iter1.stamp) {
+    if (gtk_tree_selection_get_selected(selection, &model, &iter1)) {
         GtkTreePath *path = gtk_tree_model_get_path(model, &iter1);
-
         if (gtk_tree_path_prev(path)) {
             gtk_tree_model_get_iter(model, &iter2, path);
-            gtk_tree_model_get(model, &iter1,
-                               DATA_COLUMN, &fil1,
-                               INCOMING_COLUMN, &incoming1,
-                               CLOSING_COLUMN, &closing1,
-                               -1);
-            gtk_tree_model_get(model, &iter2,
-                               DATA_COLUMN, &fil2,
-                               INCOMING_COLUMN, &incoming2,
-                               CLOSING_COLUMN, &closing2,
-                               -1);
-            gtk_list_store_set(GTK_LIST_STORE(model), &iter1,
-                               NAME_COLUMN, fil2->actual_filter->name,
-                               DATA_COLUMN, fil2,
-                               INCOMING_COLUMN, incoming2,
-                               CLOSING_COLUMN, closing2,
-                               -1);
-            gtk_list_store_set(GTK_LIST_STORE(model), &iter2,
-                               NAME_COLUMN, fil1->actual_filter->name,
-                               DATA_COLUMN, fil1,
-                               INCOMING_COLUMN, incoming1,
-                               CLOSING_COLUMN, closing1,
-                               -1);
-            p->filters_modified = TRUE;
+            fr_swap(model, &iter1, &iter2, selection, path, p);
         }
         gtk_tree_path_free(path);
     }
