@@ -30,12 +30,13 @@
 #include "save-restore.h"
 #include "pref-manager.h"
 #include "imap-server.h"
+#include "i18n.h"
 
 typedef struct _CommonDialogData CommonDialogData;
 typedef struct _FolderDialogData FolderDialogData;
 typedef struct _SubfolderDialogData SubfolderDialogData;
 
-typedef void (*CommonDialogFunc)(CommonDialogData * cdd);
+typedef gboolean (*CommonDialogFunc)(CommonDialogData * cdd);
 
 #define FOLDER_CONF_COMMON \
     GtkDialog *dialog; \
@@ -51,10 +52,10 @@ struct _CommonDialogData {
 struct _FolderDialogData {
     FOLDER_CONF_COMMON;
     BalsaServerConf bsc;
-    GtkWidget *folder_name, *port, *username, *remember,
+    GtkWidget *folder_name, *port, *username, *anonymous, *remember,
         *password, *subscribed, *list_inbox, *prefix;
     GtkWidget *use_ssl, *tls_mode;
-    GtkWidget *connection_limit, *enable_persistent, *has_bugs;
+    GtkWidget *connection_limit, *enable_persistent, *has_bugs, *use_status;
 };
 
 /* FIXME: identity_name will leak on cancelled folder edition */
@@ -62,7 +63,7 @@ struct _FolderDialogData {
 struct _SubfolderDialogData {
     FOLDER_CONF_COMMON;
     BalsaMailboxConfView *mcv;
-    GtkWidget *parent_folder, *folder_name;
+    GtkWidget *parent_folder, *folder_name, *host_label;
     gchar *old_folder, *old_parent;
     BalsaMailboxNode *parent;   /* (new) parent of the mbnode.  */
     /* Used for renaming and creation */
@@ -108,8 +109,9 @@ folder_conf_response(GtkDialog * dialog, int response,
 	    g_object_unref(cdd->mbnode);
         return;
     case GTK_RESPONSE_OK:
-        cdd->ok(cdd);
-        /* Fall over */
+        if(!cdd->ok(cdd))
+            break;
+        /* ...or fall over */
     default:
         gtk_widget_destroy(GTK_WIDGET(cdd->dialog));
         cdd->dialog = NULL;
@@ -147,6 +149,13 @@ validate_folder(GtkWidget *w, FolderDialogData * fcw)
     gtk_dialog_set_response_sensitive(fcw->dialog, GTK_RESPONSE_OK, sensitive);
 }
 
+static void
+anonymous_cb(GtkToggleButton * button, FolderDialogData * fcw)
+{
+    gtk_widget_set_sensitive(fcw->anonymous,
+                             gtk_toggle_button_get_active(button));
+}
+
 static void
 remember_cb(GtkToggleButton * button, FolderDialogData * fcw)
 {
@@ -154,7 +163,7 @@ remember_cb(GtkToggleButton * button, FolderDialogData * fcw)
                              gtk_toggle_button_get_active(button));
 }
 
-static void
+static gboolean
 folder_conf_clicked_ok(FolderDialogData * fcw)
 {
     gboolean insert;
@@ -188,7 +197,12 @@ folder_conf_clicked_ok(FolderDialogData * fcw)
     libbalsa_imap_server_set_bug
         (LIBBALSA_IMAP_SERVER(s), ISBUG_FETCH,
          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->has_bugs)));
+    libbalsa_imap_server_set_use_status
+        (LIBBALSA_IMAP_SERVER(s),
+         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->use_status)));
     libbalsa_server_set_username(s, username);
+    s->try_anonymous =
+        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->anonymous));
     s->remember_passwd =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->remember));
     libbalsa_server_set_password(s,
@@ -215,26 +229,23 @@ folder_conf_clicked_ok(FolderDialogData * fcw)
     fcw->mbnode->list_inbox =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->list_inbox));
 
-    /* Set host after all other server changes, as it triggers
-     * save-to-config for any folder or mailbox using this server. */
     libbalsa_server_set_host(s, host, 
                              balsa_server_conf_get_use_ssl(&fcw->bsc));
+    libbalsa_server_config_changed(s); /* trigger config save */
 
     if (insert) {
 	balsa_mblist_mailbox_node_append(NULL, fcw->mbnode);
         balsa_mailbox_node_append_subtree(fcw->mbnode);
         config_folder_add(fcw->mbnode, NULL);
-	g_signal_connect_swapped(s, "set-host",
+	g_signal_connect_swapped(s, "config-changed",
 		                 G_CALLBACK(config_folder_update),
 				 fcw->mbnode);
         update_mail_servers();
     } else {
         balsa_mailbox_node_rescan(fcw->mbnode);
-	/* Calling libbalsa_server_set_host already triggered an update,
-	 * so we do not need to here:
-	 * config_folder_update(fcw->mbnode); */
 	balsa_mblist_mailbox_node_redraw(fcw->mbnode);
     }
+    return TRUE;
 }
 
 /* folder_conf_imap_node:
@@ -249,6 +260,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     static FolderDialogData *fcw_new;
     LibBalsaServer *s;
     gchar *default_server;
+    int r = 0;
 
     /* Allow only one dialog per mailbox node, and one with mn == NULL
      * for creating a new folder. */
@@ -269,7 +281,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                    (_("Remote IMAP folder"),
                     GTK_WINDOW(balsa_app.main_window),
                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                    mn ? _("_Update") : _("_Create"), GTK_RESPONSE_OK,
+                    mn ? _("_Update") : _("C_reate"), GTK_RESPONSE_OK,
                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                     GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL));
     g_object_add_weak_pointer(G_OBJECT(fcw->dialog),
@@ -289,14 +301,15 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     notebook = gtk_notebook_new();
     gtk_box_pack_start(GTK_BOX(fcw->dialog->vbox),
                        notebook, TRUE, TRUE, 0);
-    table = gtk_table_new(9, 2, FALSE);
+    table = libbalsa_create_table(9, 2);
+    gtk_container_set_border_width(GTK_CONTAINER(table), 12);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table,
                              gtk_label_new_with_mnemonic(_("_Basic")));
     advanced = balsa_server_conf_get_advanced_widget(&fcw->bsc, s, 3);
     /* Limit number of connections */
     fcw->connection_limit = 
         balsa_server_conf_add_spinner
-        (&fcw->bsc, _("_Max number of connections"), 1, 40, 1,
+        (&fcw->bsc, _("_Max number of connections:"), 1, 40, 1,
          s 
          ? libbalsa_imap_server_get_max_connections(LIBBALSA_IMAP_SERVER(s))
          : 20);
@@ -316,48 +329,59 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
        libbalsa_imap_server_has_bug(LIBBALSA_IMAP_SERVER(s), ISBUG_FETCH))
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->has_bugs),
                                      TRUE);
+    fcw->use_status = 
+        balsa_server_conf_add_checkbox(&fcw->bsc,
+                                       _("Use STATUS for mailbox checking"));
+    if(s &&
+       libbalsa_imap_server_get_use_status(LIBBALSA_IMAP_SERVER(s)))
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->use_status),
+                                     TRUE);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), advanced,
                              gtk_label_new_with_mnemonic(_("_Advanced")));
 
     /* INPUT FIELD CREATION */
-    label = create_label(_("Descriptive _Name:"), table, 0);
-    fcw->folder_name = create_entry(fcw->dialog, table,
-                                   GTK_SIGNAL_FUNC(validate_folder),
-                                   fcw, 0, mn ? mn->name : NULL, 
+    label = libbalsa_create_label(_("Descriptive _Name:"), table, 0);
+    fcw->folder_name = libbalsa_create_entry(table,
+                                   G_CALLBACK(validate_folder),
+                                   fcw, r++, mn ? mn->name : NULL, 
 				   label);
 
     default_server = libbalsa_guess_imap_server();
-    label = create_label(_("_Server:"), table, 1);
-    fcw->bsc.server = create_entry(fcw->dialog, table,
-                              GTK_SIGNAL_FUNC(validate_folder),
-                              fcw, 1, s ? s->host : default_server,
+    label = libbalsa_create_label(_("_Server:"), table, 1);
+    fcw->bsc.server = libbalsa_create_entry(table,
+                              G_CALLBACK(validate_folder),
+                              fcw, r++, s ? s->host : default_server,
 			      label);
     fcw->bsc.default_ports = IMAP_DEFAULT_PORTS;
     g_free(default_server);
 
-    label= create_label(_("Use_r name:"), table, 3);
-    fcw->username = create_entry(fcw->dialog, table,
-                                GTK_SIGNAL_FUNC(validate_folder),
-                                fcw, 3, s ? s->user : g_get_user_name(), 
+    label= libbalsa_create_label(_("Use_r name:"), table, r);
+    fcw->username = libbalsa_create_entry(table,
+                                G_CALLBACK(validate_folder),
+                                fcw, r++, s ? s->user : g_get_user_name(), 
 			        label);
 
-    fcw->remember = create_check(fcw->dialog, _("_Remember password"), 
-                                table, 4, s ? s->remember_passwd : TRUE);
-    g_signal_connect(G_OBJECT(fcw->remember), "toggled",
-                     G_CALLBACK(remember_cb), fcw);
-
-    label = create_label(_("_Password:"), table, 5);
-    fcw->password = create_entry(fcw->dialog, table, NULL, NULL, 5,
+    label = libbalsa_create_label(_("_Password:"), table, r);
+    fcw->password = libbalsa_create_entry(table, NULL, NULL, r++,
 				s ? s->passwd : NULL, label);
     gtk_entry_set_visibility(GTK_ENTRY(fcw->password), FALSE);
 
-    fcw->subscribed = create_check(fcw->dialog, _("Subscribed _folders only"), 
-                                  table, 6, mn ? mn->subscribed : FALSE);
-    fcw->list_inbox = create_check(fcw->dialog, _("Always show _INBOX"), 
-                                  table, 7, mn ? mn->list_inbox : TRUE); 
+    fcw->anonymous = libbalsa_create_check(_("_Anonymous access"), 
+                                table, r++, s ? s->try_anonymous : FALSE);
+    g_signal_connect(G_OBJECT(fcw->anonymous), "toggled",
+                     G_CALLBACK(anonymous_cb), fcw);
+    fcw->remember = libbalsa_create_check(_("_Remember password"), 
+                                table, r++, s ? s->remember_passwd : TRUE);
+    g_signal_connect(G_OBJECT(fcw->remember), "toggled",
+                     G_CALLBACK(remember_cb), fcw);
 
-    label = create_label(_("Pr_efix"), table, 8);
-    fcw->prefix = create_entry(fcw->dialog, table, NULL, NULL, 8,
+    fcw->subscribed = libbalsa_create_check(_("Subscribed _folders only"), 
+                                  table, r++, mn ? mn->subscribed : FALSE);
+    fcw->list_inbox = libbalsa_create_check(_("Always show _INBOX"), 
+                                  table, r++, mn ? mn->list_inbox : TRUE); 
+
+    label = libbalsa_create_label(_("Pr_efix:"), table, r);
+    fcw->prefix = libbalsa_create_entry(table, NULL, NULL, r++,
 			      mn ? mn->dir : NULL, label);
     
     gtk_widget_show_all(GTK_WIDGET(fcw->dialog));
@@ -388,8 +412,7 @@ validate_sub_folder(GtkWidget * w, SubfolderDialogData * sdd)
      * Allow typing in the parent_folder entry box only if we already
      * have the server information in mn:
      */
-    gboolean have_server = (mn && mn->server
-			    && mn->server->type == LIBBALSA_SERVER_IMAP);
+    gboolean have_server = (mn && LIBBALSA_IS_IMAP_SERVER(mn->server));
     gtk_editable_set_editable(GTK_EDITABLE(sdd->parent_folder),
 			      have_server);
     /*
@@ -449,8 +472,11 @@ browse_button_response(GtkDialog * dialog, gint response,
         if (mbnode->dir)
             gtk_entry_set_text(GTK_ENTRY(bbd->sdd->parent_folder),
                                mbnode->dir);
+        if(mbnode->server)
+            gtk_label_set_label(GTK_LABEL(bbd->sdd->host_label),
+                                mbnode->server->host);
     }
-
+    validate_sub_folder(NULL, bbd->sdd);
     gtk_widget_set_sensitive(bbd->button, TRUE);
     gtk_widget_destroy(GTK_WIDGET(dialog));
 }
@@ -466,8 +492,7 @@ folder_selection_func(GtkTreeSelection * selection, GtkTreeModel * model,
 
     gtk_tree_model_get_iter(model, &iter, path);
     gtk_tree_model_get(model, &iter, 0, &mbnode, -1);
-    retval = (mbnode->server
-	      && mbnode->server->type == LIBBALSA_SERVER_IMAP
+    retval = (LIBBALSA_IS_IMAP_SERVER(mbnode->server)
 	      && (sdd->mbnode == NULL
 		  || sdd->mbnode->server == mbnode->server));
     g_object_unref(mbnode);
@@ -549,10 +574,11 @@ browse_button_cb(GtkWidget * widget, SubfolderDialogData * sdd)
     gtk_widget_show_all(GTK_WIDGET(dialog));
 }
 
-static void
+static gboolean
 subfolder_conf_clicked_ok(SubfolderDialogData * sdd)
 {
     gchar *parent, *folder;
+    gboolean ret = TRUE;
 
     parent =
         gtk_editable_get_chars(GTK_EDITABLE(sdd->parent_folder), 0, -1);
@@ -597,15 +623,23 @@ folder, parent);
                 gtk_widget_destroy(ask);
             }
             if (button == GTK_RESPONSE_OK) {
+                GError* err = NULL;
                 /* Close the mailbox before renaming,
                  * otherwise the rescan will try to close it
                  * under its old name.
                  */
                 balsa_window_close_mbnode(balsa_app.main_window,
                                           sdd->mbnode);
-                libbalsa_imap_rename_subfolder
-                    (LIBBALSA_MAILBOX_IMAP(sdd->mbnode->mailbox),
-                     parent, folder, sdd->mbnode->subscribed);
+                if(!libbalsa_imap_rename_subfolder
+                   (LIBBALSA_MAILBOX_IMAP(sdd->mbnode->mailbox),
+                    parent, folder, sdd->mbnode->subscribed, &err)) {
+                    balsa_information(LIBBALSA_INFORMATION_ERROR,
+                                      _("Folder rename failed. Reason: %s"),
+                                      err ? err->message : "unknown");
+                    g_clear_error(&err);
+                    ret = FALSE;
+                    goto error;
+                }
                 g_free(sdd->mbnode->dir);
                 sdd->mbnode->dir = g_strdup(parent);
 
@@ -643,16 +677,25 @@ folder, parent);
             }
         }
     } else {
+        GError *err = NULL;
         /* create and subscribe, if parent was. */
-        libbalsa_imap_new_subfolder(parent, folder,
-                                    sdd->parent->subscribed,
-                                    sdd->parent->server);
-
-        /* see it as server sees it: */
-        balsa_mailbox_node_rescan(sdd->parent);
+        if(libbalsa_imap_new_subfolder(parent, folder,
+                                       sdd->parent->subscribed,
+                                       sdd->parent->server, &err)) {
+            /* see it as server sees it: */
+            balsa_mailbox_node_rescan(sdd->parent);
+        } else {
+            balsa_information(LIBBALSA_INFORMATION_ERROR,
+                              _("Folder creation failed. Reason: %s"),
+                              err ? err->message : "unknown");
+            g_clear_error(&err);
+            ret = FALSE;
+        }
     }
+ error:
     g_free(parent);
     g_free(folder);
+    return ret;
 }
 
 /* folder_conf_imap_sub_node:
@@ -726,33 +769,41 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
 			       _("Create subfolder"));
     gtk_box_pack_start(GTK_BOX(sdd->dialog->vbox),
                        frame, TRUE, TRUE, 0);
-    table = gtk_table_new(3, 3, FALSE);
+    table = libbalsa_create_table(3, 3);
+    gtk_container_set_border_width(GTK_CONTAINER(table), 12);
     gtk_container_add(GTK_CONTAINER(frame), table);
  
     /* INPUT FIELD CREATION */
-    label= create_label(_("_Folder name:"), table, 0);
-    sdd->folder_name = create_entry(sdd->dialog, table,
-                                   GTK_SIGNAL_FUNC(validate_sub_folder),
+    label= libbalsa_create_label(_("_Folder name:"), table, 0);
+    sdd->folder_name = libbalsa_create_entry(table,
+                                   G_CALLBACK(validate_sub_folder),
 				   sdd, 0, sdd->old_folder, label);
 
-    subtable = gtk_table_new(1, 3, FALSE);
-    label = create_label(_("_Subfolder of:"), table, 1);
-    sdd->parent_folder = create_entry(sdd->dialog, subtable,
-                                     GTK_SIGNAL_FUNC(validate_sub_folder),
-				     sdd, 0, sdd->old_parent, label);
-
+    subtable = libbalsa_create_table(1, 3);
+    label = libbalsa_create_label(_("Host:"), table, 1);
+    sdd->host_label = 
+        gtk_label_new(sdd->mbnode && sdd->mbnode->server
+                      ? sdd->mbnode->server->host : "");
+    gtk_table_attach(GTK_TABLE(subtable), sdd->host_label, 1, 2, 0, 1,
+		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 5);
     button = gtk_button_new_with_mnemonic(_("_Browse..."));
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(browse_button_cb), (gpointer) sdd);
     gtk_table_attach(GTK_TABLE(subtable), button, 2, 3, 0, 1,
-	GTK_EXPAND | GTK_FILL, GTK_FILL, 5, 5);
+	GTK_FILL, GTK_FILL, 5, 5);
     gtk_table_attach(GTK_TABLE(table), subtable, 1, 2, 1, 2,
 	GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 5);
 
+    label = libbalsa_create_label(_("_Subfolder of:"), table, 2);
+    sdd->parent_folder = libbalsa_create_entry(table,
+                                     G_CALLBACK(validate_sub_folder),
+				     sdd, 2, sdd->old_parent, label);
+
+
     if (mn)
         sdd->mcv = mailbox_conf_view_new(mn->mailbox,
                                          GTK_WINDOW(sdd->dialog),
-                                         table, 2);
+                                         table, 3);
 
     gtk_widget_show_all(GTK_WIDGET(sdd->dialog));
 
