@@ -27,8 +27,10 @@
 #include "config.h"
 
 #include <gnome.h>
+#include <string.h>
 
 #include "libbalsa.h"
+#include "i18n.h"
 #include "balsa-app.h"
 #include "information-dialog.h"
 
@@ -193,7 +195,7 @@ balsa_information_list_new(void)
     gtk_text_view_set_left_margin(view, 2);
     gtk_text_view_set_indent(view, -12);
     gtk_text_view_set_right_margin(view, 2);
-    gtk_text_view_set_wrap_mode(view, GTK_WRAP_WORD);
+    gtk_text_view_set_wrap_mode(view, GTK_WRAP_WORD_CHAR);
 
     return GTK_WIDGET(view);
 }
@@ -268,37 +270,51 @@ balsa_information_list(GtkWindow *parent, LibBalsaInformationType type,
                                  gtk_text_buffer_get_insert(buffer),
                                  0, FALSE, 0, 0);
 
-    if (balsa_app.appbar)
-	gnome_appbar_set_status(balsa_app.appbar, msg);
+    if (balsa_app.appbar) {
+        gchar *line = g_strdup(msg);
+        g_strdelimit(line, "\r\n", ' ');
+	gnome_appbar_set_status(balsa_app.appbar, line);
+        g_free(line);
+    }
 }
 
 static guint bar_timeout_id = 0;
-static unsigned push_cnt = 0;
 static gboolean
 status_bar_refresh(gpointer data)
 {
     gdk_threads_enter();
-    if (balsa_app.appbar) {
+
+    if (balsa_app.appbar)
         gnome_appbar_pop(balsa_app.appbar);
-        push_cnt--;
-        if(push_cnt == 0)
-            bar_timeout_id = 0; /* FIXME: thread locking here! */
-    }
+
+    bar_timeout_id = 0;
+
     gdk_threads_leave();
-    return push_cnt != 0;
+
+    return FALSE;
 }
 
 static void
 balsa_information_bar(GtkWindow *parent, LibBalsaInformationType type,
                       const char *msg)
 {
+    gchar *line;
+
     if (!balsa_app.appbar)
         return;
-    /* we used to have _set_status() here but it got over-written by
-       set_default(). _push() seems to be more persistent. */
-    gnome_appbar_push(balsa_app.appbar, msg); push_cnt++;
-    if(!bar_timeout_id)
-        bar_timeout_id = g_timeout_add(4000, status_bar_refresh, NULL);
+
+    /* First clear any current message. */
+    if (bar_timeout_id) {
+        gnome_appbar_pop(balsa_app.appbar);
+        g_source_remove(bar_timeout_id);
+    }
+
+    line = g_strdup(msg);
+    g_strdelimit(line, "\r\n", ' ');
+    gnome_appbar_push(balsa_app.appbar, line);
+    g_free(line);
+
+    bar_timeout_id = g_timeout_add(4000, status_bar_refresh, NULL);
 }
 
 static void 
