@@ -50,7 +50,7 @@ struct _BalsaWindow {
     GtkWidget *hpaned;
     GtkWidget *vpaned;
     GtkWidget *current_index;
-    GtkWidget *filter_label;
+    GtkWidget *filter_choice;
     LibBalsaMessage *current_message;
 };
 
@@ -71,19 +71,24 @@ typedef enum {
     TRASH_CHECK  /* uncertain, better check */
 } TrashState;
 
+enum {
+    FILTER_SENDER  = 0,
+    FILTER_RECIPIENT = 1
+};
 
 GtkType balsa_window_get_type(void);
 GtkWidget *balsa_window_new(void);
 GtkWidget *balsa_window_find_current_index(BalsaWindow * window);
 void balsa_window_enable_mailbox_menus(BalsaWindow * window,
 				       BalsaIndex * index);
-LibBalsaCondition* balsa_window_get_view_filter(BalsaWindow *window);
+void balsa_window_update_book_menus(BalsaWindow *window);
+LibBalsaCondition* balsa_window_get_view_filter(BalsaWindow *window,
+                                                gboolean flags_only);
 void balsa_window_refresh(BalsaWindow * window);
 void balsa_window_open_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void balsa_window_close_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void balsa_identities_changed(BalsaWindow *bw);
 
-void balsa_window_set_filter_label(BalsaWindow * window, gboolean to_field);
 void balsa_window_update_tab(BalsaMailboxNode * mbnode);
 void enable_empty_trash(BalsaWindow * window, TrashState status);
 void balsa_window_enable_continue(BalsaWindow * window);
@@ -96,18 +101,17 @@ void check_new_messages_count(LibBalsaMailbox * mailbox, gboolean notify);
 void empty_trash(BalsaWindow * window);
 void update_view_menu(BalsaWindow * window);
 BalsaToolbarModel *balsa_window_get_toolbar_model(void);
+void balsa_window_select_all(GtkWindow * window);
+gboolean balsa_window_next_unread(BalsaWindow * window);
 
 /* functions to manipulate the progress bars of the window */
 void balsa_window_increase_activity(BalsaWindow* window);
 void balsa_window_decrease_activity(BalsaWindow* window);
-gboolean balsa_window_setup_progress(BalsaWindow* window, gfloat upper_bound);
+gboolean balsa_window_setup_progress(BalsaWindow * window,
+                                     const gchar * text);
 void balsa_window_clear_progress(BalsaWindow* window);
-#ifdef BALSA_USE_THREADS
-/* the increment model was not designed for threading and does not work. */
-#define balsa_window_increment_progress(arg)
-#else
-void balsa_window_increment_progress(BalsaWindow* window);
-#endif
+void balsa_window_increment_progress(BalsaWindow * window,
+                                     gdouble fraction, gboolean flush);
 
 #if defined(__FILE__) && defined(__LINE__)
 # ifdef __FUNCTION__
