@@ -23,26 +23,13 @@
 
 #include <string.h>
 #include <gnome.h>
+#include "i18n.h"
 
-#include "libbalsa.h"
-
-#include "address-book.h"
 #include "balsa-app.h"
 #include "balsa-icons.h"
-#include "balsa-index.h"
-#include "balsa-mblist.h"
-#include "balsa-message.h"
-#include "folder-conf.h"
-#include "mailbox-conf.h"
 #include "main-window.h"
 #include "message-window.h"
-#include "pref-manager.h"
-#include "print.h"
 #include "sendmsg-window.h"
-#include "store-address.h"
-#include "save-restore.h"
-
-#include "libinit_balsa/init_balsa.h"
 
 #include "toolbar-prefs.h"
 #include "toolbar-factory.h"
@@ -90,7 +77,7 @@ button_data toolbar_buttons[]={
     {"", N_("Separator"), "", 0},
     {BALSA_PIXMAP_RECEIVE, N_("Check"),
      N_("Check for new email"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_NEW, N_("Compose"),
+    {BALSA_PIXMAP_COMPOSE, N_("Compose"),
      N_("Compose message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_CONTINUE, N_("Continue"),
      N_("Continue message"), TOOLBAR_BUTTON_TYPE_BUTTON},
@@ -114,11 +101,11 @@ button_data toolbar_buttons[]={
      N_("View previous part of message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_NEXT_PART, N_("Next\npart"),
      N_("View next part of message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_TRASH, N_("Trash /\nDelete"),
+    {GTK_STOCK_DELETE, N_("Trash /\nDelete"),
      N_("Move the current message to trash"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_POSTPONE, N_("Postpone"),
      N_("Postpone current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_PRINT, N_("Print"),
+    {GTK_STOCK_PRINT, N_("Print"),
      N_("Print current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SEND, N_("Send"),
      N_("Send this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
@@ -126,23 +113,27 @@ button_data toolbar_buttons[]={
      N_("Send and Receive messages"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_ATTACHMENT, N_("Attach"),
      N_("Add attachments to this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_SAVE, N_("Save"),
+    {GTK_STOCK_SAVE, N_("Save"),
      N_("Save the current item"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_IDENTITY, N_("Identity"),
      N_("Set identity to use for this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {GTK_STOCK_SPELL_CHECK, N_("Spelling"),
+#if HAVE_GTKSPELL
+     N_("Toggle spell checker"), TOOLBAR_BUTTON_TYPE_TOGGLE},
+#else                           /* HAVE_GTKSPELL */
      N_("Run a spell check"), TOOLBAR_BUTTON_TYPE_BUTTON},
+#endif                          /* HAVE_GTKSPELL */
     {GTK_STOCK_CLOSE, N_("Close"), 
      N_("Close the compose window"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_MARKED_NEW, N_("Toggle\nnew"),
      N_("Toggle new message flag"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_MARKED_ALL, N_("Mark all"),
+    {BALSA_PIXMAP_MARK_ALL, N_("Mark all"),
      N_("Mark all messages in current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SHOW_HEADERS, N_("All\nheaders"),
      N_("Show all headers"), TOOLBAR_BUTTON_TYPE_TOGGLE},
     {BALSA_PIXMAP_TRASH_EMPTY, N_("Empty Trash"),
      N_("Delete messages from the Trash mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_CLOSE_MBOX, N_("Close"),
+    {GTK_STOCK_CANCEL, N_("Close"),
      N_("Close current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SHOW_PREVIEW, N_("Msg Preview"),
      N_("Show preview pane"), TOOLBAR_BUTTON_TYPE_TOGGLE},
@@ -156,38 +147,35 @@ button_data toolbar_buttons[]={
     N_("Undo most recent change"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {GTK_STOCK_REDO, N_("Redo"),
     N_("Redo most recent change"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    { GTK_STOCK_CLEAR, N_("Expunge\nDeleted"),
+      N_("Expunge messages marked as deleted"), TOOLBAR_BUTTON_TYPE_BUTTON }
 };
 
 const int toolbar_button_count =
     sizeof(toolbar_buttons) / sizeof(button_data);
 
 /* Public methods. */
+const gchar *
+balsa_toolbar_sanitize_id(const gchar *id)
+{
+    gint button = get_toolbar_button_index(id);
+
+    if (button >= 0)
+	return toolbar_buttons[button].pixmap_id;
+    else
+	return NULL;
+}
 
 /* this should go to GTK because it modifies its internal structures. */
 void
 balsa_toolbar_remove_all(GtkWidget * widget)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     GList *child, *children;
     
     children = gtk_container_get_children(GTK_CONTAINER(widget));
     for (child = children; child; child = child->next)
         gtk_widget_destroy(child->data);
     g_list_free(children);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    GtkToolbar *toolbar = GTK_TOOLBAR(widget);
-    
-    /* we must be extremely careful here. unparenting used in original
-     * code modifies the children list with gtk+2.4 so we have to
-     * do toolbar cleanup differently. */
-    while (toolbar->children) {
-	GtkToolbarChild *child = toolbar->children->data;
-	if (child->type == GTK_TOOLBAR_CHILD_SPACE)
-	    gtk_toolbar_remove_space(toolbar, 0);
-	else
-	    gtk_container_remove(GTK_CONTAINER(widget), child->widget);
-    }
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 }
 
 /* update_all_toolbars:
@@ -242,9 +230,11 @@ void
 balsa_toolbar_model_insert_icon(BalsaToolbarModel * model, gchar * icon,
                                 gint position)
 {
-    if (get_toolbar_button_index(icon) >= 0)
+    const gchar* real_button = balsa_toolbar_sanitize_id(icon);
+
+    if (real_button)
         *model->current =
-            g_slist_insert(*model->current, g_strdup(icon), position);
+            g_slist_insert(*model->current, g_strdup(real_button), position);
     else
         g_warning(_("Unknown toolbar icon \"%s\""), icon);
 }
@@ -311,19 +301,11 @@ balsa_toolbar_refresh(GtkWidget * toolbar)
         BalsaToolbarIcon *bti;
         gint button;
         gchar *text, *tmp;
-#if GTK_CHECK_VERSION(2, 4, 0)
 	GtkToolItem *tool_item;
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-        gint type;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 
         if (!*icon) {
-#if GTK_CHECK_VERSION(2, 4, 0)
             gtk_toolbar_insert(GTK_TOOLBAR(toolbar),
                                gtk_separator_tool_item_new(), -1);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-            gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
             continue;
         }
 
@@ -334,6 +316,9 @@ balsa_toolbar_refresh(GtkWidget * toolbar)
             continue;
 
         tmp = _(toolbar_buttons[button].button_text);
+        if (toolbar_buttons[button].pixmap_id == BALSA_PIXMAP_SEND
+            && balsa_app.always_queue_sent_mail)
+            tmp = _("Queue");
         if (balsa_app.toolbar_wrap_button_text) {
             /* Make sure all buttons have the same number of lines of
              * text (1 or 2), to keep icons aligned */
@@ -346,13 +331,7 @@ balsa_toolbar_refresh(GtkWidget * toolbar)
         }
 
         switch (toolbar_buttons[button].type) {
-#if !GTK_CHECK_VERSION(2, 4, 0)
-        case TOOLBAR_BUTTON_TYPE_RADIO:
-            type = GTK_TOOLBAR_CHILD_RADIOBUTTON;
-            break;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
         case TOOLBAR_BUTTON_TYPE_TOGGLE:
-#if GTK_CHECK_VERSION(2, 4, 0)
 	    tool_item = gtk_toggle_tool_button_new_from_stock(icon);
             gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON
                                               (tool_item), bti->active);
@@ -361,59 +340,35 @@ balsa_toolbar_refresh(GtkWidget * toolbar)
 	    if (bti->callback)
 		g_signal_connect(G_OBJECT(tool_item), "toggled",
                                  G_CALLBACK(bti->callback), bti->user_data);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-            type = GTK_TOOLBAR_CHILD_TOGGLEBUTTON;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
             break;
         case TOOLBAR_BUTTON_TYPE_BUTTON:
         default:
-#if GTK_CHECK_VERSION(2, 4, 0)
 	    tool_item = gtk_tool_button_new_from_stock(icon);
 	    if (bti->callback)
 		g_signal_connect(G_OBJECT(tool_item), "clicked",
                                  G_CALLBACK(bti->callback), bti->user_data);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-            type = GTK_TOOLBAR_CHILD_BUTTON;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
             break;
         }
-#if GTK_CHECK_VERSION(2, 4, 0)
 	gtk_tool_button_set_label(GTK_TOOL_BUTTON(tool_item), text);
+        g_free(text);
+
+        text = _(toolbar_buttons[button].help_text);
+        if (toolbar_buttons[button].pixmap_id == BALSA_PIXMAP_SEND
+            && balsa_app.always_queue_sent_mail)
+            text = _("Queue this message for sending");
         gtk_tool_item_set_tooltip(tool_item,
                                   GTK_TOOLBAR(toolbar)->tooltips,
-                                  _(toolbar_buttons[button].help_text),
-                                  _(toolbar_buttons[button].help_text));
+                                  text, text);
+
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tool_item, -1);
 	bti->widget = GTK_WIDGET(tool_item);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-        bti->widget =
-            gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), type, NULL,
-                                       text,
-                                       _(toolbar_buttons[button].help_text),
-                                       _(toolbar_buttons[button].help_text),
-                                       gtk_image_new_from_stock(icon,
-                                           GTK_ICON_SIZE_LARGE_TOOLBAR),
-                                       GTK_SIGNAL_FUNC(bti->callback),
-                                       bti->user_data);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 	g_object_add_weak_pointer(G_OBJECT(bti->widget),
 				  (gpointer) &bti->widget);
         GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(bti->widget), GTK_CAN_FOCUS);
-        g_free(text);
         gtk_widget_set_sensitive(bti->widget, bti->sensitive);
-#if !GTK_CHECK_VERSION(2, 4, 0)
-        if (type == GTK_TOOLBAR_CHILD_TOGGLEBUTTON) {
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bti->widget),
-                                         bti->active);
-            g_signal_connect(G_OBJECT(bti->widget), "toggled",
-                             G_CALLBACK(bt_button_toggled), bti);
-        }
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     }
 
-#if GTK_CHECK_VERSION(2, 4, 0)
     gtk_widget_show_all(toolbar);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     parent = gtk_widget_get_parent(toolbar);
     if (parent)
         gtk_widget_queue_resize(parent);
@@ -488,13 +443,8 @@ balsa_toolbar_set_button_active(GtkWidget * toolbar, const gchar * icon,
 
     bti->active = active;
     if (bti->widget)
-#if GTK_CHECK_VERSION(2, 4, 0)
         gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON
                                           (bti->widget), active);
-#else                           /* GTK_CHECK_VERSION(2, 4, 0) */
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bti->widget),
-                                     active);
-#endif                          /* GTK_CHECK_VERSION(2, 4, 0) */
 }
 
 /* Signal handlers. */
@@ -503,12 +453,8 @@ balsa_toolbar_set_button_active(GtkWidget * toolbar, const gchar * icon,
 static void
 bt_button_toggled(GtkWidget * button, BalsaToolbarIcon * bti)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     bti->active =
         gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(button));
-#else                           /* GTK_CHECK_VERSION(2, 4, 0) */
-    bti->active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
-#endif                          /* GTK_CHECK_VERSION(2, 4, 0) */
 }
 
 /* Handler for the "destroy" signal. */
